@@ -21,8 +21,14 @@
  */
 
 
-#include <gstfakesink.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
+#include "gstfakesink.h"
+
+GST_DEBUG_CATEGORY (gst_fakesink_debug);
+#define GST_CAT_DEFAULT gst_fakesink_debug
 
 GstElementDetails gst_fakesink_details = {
   "Fake Sink",
@@ -44,10 +50,12 @@ enum {
 
 enum {
   ARG_0,
+  ARG_STATE_ERROR,
   ARG_NUM_SINKS,
   ARG_SILENT,
   ARG_DUMP,
   ARG_SYNC,
+  ARG_SIGNAL_HANDOFFS,
   ARG_LAST_MESSAGE,
 };
 
@@ -55,9 +63,29 @@ GST_PAD_TEMPLATE_FACTORY (fakesink_sink_factory,
   "sink%d",
   GST_PAD_SINK,
   GST_PAD_REQUEST,
-  NULL                  /* no caps */
+  GST_CAPS_ANY
 );
 
+#define GST_TYPE_FAKESINK_STATE_ERROR (gst_fakesink_state_error_get_type())
+static GType
+gst_fakesink_state_error_get_type (void)
+{
+  static GType fakesink_state_error_type = 0;
+  static GEnumValue fakesink_state_error[] = {
+    { FAKESINK_STATE_ERROR_NONE,           "0", "No state change errors"},
+    { FAKESINK_STATE_ERROR_NULL_READY,     "1", "Fail state change from NULL to READY"},
+    { FAKESINK_STATE_ERROR_READY_PAUSED,   "2", "Fail state change from READY to PAUSED"},
+    { FAKESINK_STATE_ERROR_PAUSED_PLAYING, "3", "Fail state change from PAUSED to PLAYING"},
+    { FAKESINK_STATE_ERROR_PLAYING_PAUSED, "4", "Fail state change from PLAYING to PAUSED"},
+    { FAKESINK_STATE_ERROR_PAUSED_READY  , "5", "Fail state change from PAUSED to READY"},
+    { FAKESINK_STATE_ERROR_READY_NULL    , "6", "Fail state change from READY to NULL"},
+    {0, NULL, NULL},
+  };
+  if (!fakesink_state_error_type) {
+    fakesink_state_error_type = g_enum_register_static ("GstFakeSinkStateError", fakesink_state_error);
+  }
+  return fakesink_state_error_type;
+}
 
 static void	gst_fakesink_class_init		(GstFakeSinkClass *klass);
 static void	gst_fakesink_init		(GstFakeSink *fakesink);
@@ -71,7 +99,10 @@ static void	gst_fakesink_set_property	(GObject *object, guint prop_id,
 static void	gst_fakesink_get_property	(GObject *object, guint prop_id, 
 						 GValue *value, GParamSpec *pspec);
 
-static void	gst_fakesink_chain		(GstPad *pad, GstBuffer *buf);
+static GstElementStateReturn
+		gst_fakesink_change_state 	(GstElement *element);
+
+static void	gst_fakesink_chain		(GstPad *pad, GstData *_data);
 
 static GstElementClass *parent_class = NULL;
 static guint gst_fakesink_signals[LAST_SIGNAL] = { 0 };
@@ -111,21 +142,27 @@ gst_fakesink_class_init (GstFakeSinkClass *klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NUM_SINKS,
     g_param_spec_int ("num_sinks", "Number of sinks", "The number of sinkpads",
                       1, G_MAXINT, 1, G_PARAM_READABLE)); 
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_STATE_ERROR,
+    g_param_spec_enum ("state_error", "State Error", "Generate a state change error",
+                       GST_TYPE_FAKESINK_STATE_ERROR, FAKESINK_STATE_ERROR_NONE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LAST_MESSAGE,
     g_param_spec_string ("last_message", "Last Message", "The message describing current status",
                          NULL, G_PARAM_READABLE));
-  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_SYNC,
-    g_param_spec_boolean("sync","Sync","Sync on the clock",
-                         FALSE, G_PARAM_READWRITE)); /* CHECKME */
-
-  gst_element_class_install_std_props (
-	  GST_ELEMENT_CLASS (klass),
-	  "silent", ARG_SILENT, G_PARAM_READWRITE,
-	  "dump",   ARG_DUMP,   G_PARAM_READWRITE,
-	  NULL);
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SYNC,
+    g_param_spec_boolean ("sync", "Sync", "Sync on the clock",
+                          FALSE, G_PARAM_READWRITE)); 
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SIGNAL_HANDOFFS,
+    g_param_spec_boolean ("signal-handoffs", "Signal handoffs", "Send a signal before unreffing the buffer",
+                          FALSE, G_PARAM_READWRITE)); 
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SILENT,
+    g_param_spec_boolean ("silent", "Silent", "Don't produce last_message events",
+                          FALSE, G_PARAM_READWRITE)); 
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DUMP,
+    g_param_spec_boolean ("dump", "Dump", "Dump received bytes to stdout",
+                          FALSE, G_PARAM_READWRITE)); 
 
   gst_fakesink_signals[SIGNAL_HANDOFF] =
-    g_signal_new ("handoff", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
+    g_signal_new ("handoff", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
                     G_STRUCT_OFFSET (GstFakeSinkClass, handoff), NULL, NULL,
                     g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE, 1,
                     G_TYPE_POINTER);
@@ -134,12 +171,15 @@ gst_fakesink_class_init (GstFakeSinkClass *klass)
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_fakesink_get_property);
 
   gstelement_class->request_new_pad = GST_DEBUG_FUNCPTR (gst_fakesink_request_new_pad);
+  gstelement_class->set_clock       = GST_DEBUG_FUNCPTR (gst_fakesink_set_clock);
+  gstelement_class->change_state    = GST_DEBUG_FUNCPTR (gst_fakesink_change_state);
 }
 
 static void 
 gst_fakesink_init (GstFakeSink *fakesink) 
 {
   GstPad *pad;
+
   pad = gst_pad_new ("sink", GST_PAD_SINK);
   gst_element_add_pad (GST_ELEMENT (fakesink), pad);
   gst_pad_set_chain_function (pad, GST_DEBUG_FUNCPTR (gst_fakesink_chain));
@@ -148,8 +188,8 @@ gst_fakesink_init (GstFakeSink *fakesink)
   fakesink->dump = FALSE;
   fakesink->sync = FALSE;
   fakesink->last_message = NULL;
-
-  GST_ELEMENT (fakesink)->setclockfunc    = gst_fakesink_set_clock;
+  fakesink->state_error = FAKESINK_STATE_ERROR_NONE;
+  fakesink->signal_handoffs = FALSE;
 
   GST_FLAG_SET (fakesink, GST_ELEMENT_EVENT_AWARE);
 }
@@ -202,12 +242,18 @@ gst_fakesink_set_property (GObject *object, guint prop_id, const GValue *value,
     case ARG_SILENT:
       sink->silent = g_value_get_boolean (value);
       break;
+    case ARG_STATE_ERROR:
+      sink->state_error = g_value_get_enum (value);
+      break;
     case ARG_DUMP:
       sink->dump = g_value_get_boolean (value);
       break;
     case ARG_SYNC:
       sink->sync = g_value_get_boolean (value);
       break;
+    case ARG_SIGNAL_HANDOFFS:
+      sink->signal_handoffs = g_value_get_boolean (value);
+      break;
     default:
       break;
   }
@@ -227,6 +273,9 @@ gst_fakesink_get_property (GObject *object, guint prop_id, GValue *value, GParam
     case ARG_NUM_SINKS:
       g_value_set_int (value, GST_ELEMENT (sink)->numsinkpads);
       break;
+    case ARG_STATE_ERROR:
+      g_value_set_enum (value, sink->state_error);
+      break;
     case ARG_SILENT:
       g_value_set_boolean (value, sink->silent);
       break;
@@ -236,6 +285,9 @@ gst_fakesink_get_property (GObject *object, guint prop_id, GValue *value, GParam
     case ARG_SYNC:
       g_value_set_boolean (value, sink->sync);
       break;
+    case ARG_SIGNAL_HANDOFFS:
+      g_value_set_boolean (value, sink->signal_handoffs);
+      break;
     case ARG_LAST_MESSAGE:
       g_value_set_string (value, sink->last_message);
       break;
@@ -246,18 +298,28 @@ gst_fakesink_get_property (GObject *object, guint prop_id, GValue *value, GParam
 }
 
 static void 
-gst_fakesink_chain (GstPad *pad, GstBuffer *buf) 
+gst_fakesink_chain (GstPad *pad, GstData *_data) 
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstFakeSink *fakesink;
 
   g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (buf != NULL);
 
-  fakesink = GST_FAKESINK (gst_pad_get_parent (pad));
+  fakesink = GST_FAKESINK (GST_OBJECT_PARENT (pad));
 
   if (GST_IS_EVENT (buf)) {
     GstEvent *event = GST_EVENT (buf);
+    
+    if (!fakesink->silent) { 
+      g_free (fakesink->last_message);
+
+      fakesink->last_message = g_strdup_printf ("chain   ******* (%s:%s)E (type: %d) %p",
+		GST_DEBUG_PAD_NAME (pad), GST_EVENT_TYPE (event), event);
+    
+      g_object_notify (G_OBJECT (fakesink), "last_message");
+    }
 	  
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_DISCONTINUOUS:
@@ -273,19 +335,30 @@ gst_fakesink_chain (GstPad *pad, GstBuffer *buf)
   }
 
   if (fakesink->sync && fakesink->clock) { 
-    gst_element_clock_wait (GST_ELEMENT (fakesink), fakesink->clock, GST_BUFFER_TIMESTAMP (buf), NULL);
+    GstClockID id = gst_clock_new_single_shot_id (fakesink->clock, GST_BUFFER_TIMESTAMP (buf));
+
+    gst_element_clock_wait (GST_ELEMENT (fakesink), id, NULL);
+    gst_clock_id_free (id);
   }
 
   if (!fakesink->silent) { 
     g_free (fakesink->last_message);
 
-    fakesink->last_message = g_strdup_printf ("chain   ******* (%s:%s)< (%d bytes, %lld) %p",
-		GST_DEBUG_PAD_NAME (pad), GST_BUFFER_SIZE (buf), GST_BUFFER_TIMESTAMP (buf), buf);
+    fakesink->last_message = g_strdup_printf ("chain   ******* (%s:%s)< (%d bytes, timestamp: %"
+                                              G_GINT64_FORMAT ", duration: %" 
+					      G_GINT64_FORMAT ", offset: %" 
+					      G_GINT64_FORMAT ", flags: %d) %p",
+		GST_DEBUG_PAD_NAME (pad), GST_BUFFER_SIZE (buf), 
+		GST_BUFFER_TIMESTAMP (buf), 
+		GST_BUFFER_DURATION (buf), 
+		GST_BUFFER_OFFSET (buf), 
+		GST_BUFFER_FLAGS (buf), buf);
     
     g_object_notify (G_OBJECT (fakesink), "last_message");
   }
 
-  g_signal_emit (G_OBJECT (fakesink), gst_fakesink_signals[SIGNAL_HANDOFF], 0, buf, pad);
+  if (fakesink->signal_handoffs)
+    g_signal_emit (G_OBJECT (fakesink), gst_fakesink_signals[SIGNAL_HANDOFF], 0, buf, pad);
 
   if (fakesink->dump) {
     gst_util_dump_mem (GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
@@ -294,6 +367,50 @@ gst_fakesink_chain (GstPad *pad, GstBuffer *buf)
   gst_buffer_unref (buf);
 }
 
+static GstElementStateReturn
+gst_fakesink_change_state (GstElement *element)
+{
+  GstFakeSink *fakesink = GST_FAKESINK (element);
+
+  switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_NULL_TO_READY:
+      if (fakesink->state_error == FAKESINK_STATE_ERROR_NULL_READY)
+	goto error;
+      break;
+    case GST_STATE_READY_TO_PAUSED:
+      if (fakesink->state_error == FAKESINK_STATE_ERROR_READY_PAUSED)
+	goto error;
+      break;
+    case GST_STATE_PAUSED_TO_PLAYING:
+      if (fakesink->state_error == FAKESINK_STATE_ERROR_PAUSED_PLAYING)
+	goto error;
+      break;
+    case GST_STATE_PLAYING_TO_PAUSED:
+      if (fakesink->state_error == FAKESINK_STATE_ERROR_PLAYING_PAUSED)
+	goto error;
+      break;
+    case GST_STATE_PAUSED_TO_READY:
+      if (fakesink->state_error == FAKESINK_STATE_ERROR_PAUSED_READY)
+	goto error;
+      break;
+    case GST_STATE_READY_TO_NULL:
+      if (fakesink->state_error == FAKESINK_STATE_ERROR_READY_NULL)
+	goto error;
+      g_free (fakesink->last_message);
+      fakesink->last_message = NULL;
+      break;
+  }
+
+  if (GST_ELEMENT_CLASS (parent_class)->change_state)
+    return GST_ELEMENT_CLASS (parent_class)->change_state (element);
+
+  return GST_STATE_SUCCESS;
+
+error:
+  gst_element_error (element, "failed state change as requested");
+  return GST_STATE_FAILURE;
+}
+
 gboolean
 gst_fakesink_factory_init (GstElementFactory *factory)
 {
