@@ -24,12 +24,19 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gstfakesrc.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
+#include "gstfakesrc.h"
 
 #define DEFAULT_SIZEMIN		0
 #define DEFAULT_SIZEMAX		4096
 #define DEFAULT_PARENTSIZE	4096*10
 
+GST_DEBUG_CATEGORY (gst_fakesrc_debug);
+#define GST_CAT_DEFAULT gst_fakesrc_debug
+
 GstElementDetails gst_fakesrc_details = {
   "Fake Source",
   "Source",
@@ -62,6 +69,7 @@ enum {
   ARG_PATTERN,
   ARG_NUM_BUFFERS,
   ARG_EOS,
+  ARG_SIGNAL_HANDOFFS,
   ARG_SILENT,
   ARG_DUMP,
   ARG_PARENTSIZE,
@@ -72,7 +80,7 @@ GST_PAD_TEMPLATE_FACTORY (fakesrc_src_factory,
   "src%d",
   GST_PAD_SRC,
   GST_PAD_REQUEST,
-  NULL                  /* no caps */
+  GST_CAPS_ANY
 );
 
 #define GST_TYPE_FAKESRC_OUTPUT (gst_fakesrc_output_get_type())
@@ -153,7 +161,7 @@ gst_fakesrc_filltype_get_type (void)
 static void		gst_fakesrc_class_init		(GstFakeSrcClass *klass);
 static void		gst_fakesrc_init		(GstFakeSrc *fakesrc);
 
-static GstPad* 		gst_fakesrc_request_new_pad 	(GstElement *element, GstPadTemplate *templ);
+static GstPad* 		gst_fakesrc_request_new_pad 	(GstElement *element, GstPadTemplate *templ,const gchar *unused);
 static void 		gst_fakesrc_update_functions 	(GstFakeSrc *src);
 static void		gst_fakesrc_set_property	(GObject *object, guint prop_id, 
 							 const GValue *value, GParamSpec *pspec);
@@ -162,7 +170,7 @@ static void		gst_fakesrc_get_property	(GObject *object, guint prop_id,
 
 static GstElementStateReturn gst_fakesrc_change_state 	(GstElement *element);
 
-static GstBuffer*	gst_fakesrc_get			(GstPad *pad);
+static GstData*	gst_fakesrc_get			(GstPad *pad);
 static void 		gst_fakesrc_loop		(GstElement *element);
 
 static GstElementClass *parent_class = NULL;
@@ -243,6 +251,9 @@ gst_fakesrc_class_init (GstFakeSrcClass *klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SILENT,
     g_param_spec_boolean ("silent", "Silent", "Don't produce last_message events",
                           FALSE, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SIGNAL_HANDOFFS,
+    g_param_spec_boolean ("signal-handoffs", "Signal handoffs", "Send a signal before pushing the buffer",
+                          FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DUMP,
     g_param_spec_boolean ("dump", "Dump", "Dump produced bytes to stdout",
                           FALSE, G_PARAM_READWRITE));
@@ -279,6 +290,7 @@ gst_fakesrc_init (GstFakeSrc *fakesrc)
   fakesrc->rt_num_buffers = -1;
   fakesrc->buffer_count = 0;
   fakesrc->silent = FALSE;
+  fakesrc->signal_handoffs = FALSE;
   fakesrc->dump = FALSE;
   fakesrc->pattern_byte = 0x00;
   fakesrc->need_flush = FALSE;
@@ -293,7 +305,7 @@ gst_fakesrc_init (GstFakeSrc *fakesrc)
 }
 
 static GstPad*
-gst_fakesrc_request_new_pad (GstElement *element, GstPadTemplate *templ)
+gst_fakesrc_request_new_pad (GstElement *element, GstPadTemplate *templ, const gchar *unused)
 {
   gchar *name;
   GstPad *srcpad;
@@ -319,34 +331,34 @@ gst_fakesrc_request_new_pad (GstElement *element, GstPadTemplate *templ)
   return srcpad;
 }
 
-GST_FORMATS_FUNCTION (gst_fakesrc_get_formats,
+GST_PAD_FORMATS_FUNCTION (gst_fakesrc_get_formats,
   GST_FORMAT_DEFAULT
 )
 
 GST_PAD_QUERY_TYPE_FUNCTION (gst_fakesrc_get_query_types,
-  GST_PAD_QUERY_TOTAL,
-  GST_PAD_QUERY_POSITION,
-  GST_PAD_QUERY_START,
-  GST_PAD_QUERY_SEGMENT_END
+  GST_QUERY_TOTAL,
+  GST_QUERY_POSITION,
+  GST_QUERY_START,
+  GST_QUERY_SEGMENT_END
 ) 
 
 static gboolean
-gst_fakesrc_query (GstPad *pad, GstPadQueryType type,
+gst_fakesrc_query (GstPad *pad, GstQueryType type,
 	           GstFormat *format, gint64 *value)
 {
   GstFakeSrc *src = GST_FAKESRC (GST_PAD_PARENT (pad));
 
   switch (type) {
-    case GST_PAD_QUERY_TOTAL:
+    case GST_QUERY_TOTAL:
       *value = src->num_buffers;
       break;
-    case GST_PAD_QUERY_POSITION:
+    case GST_QUERY_POSITION:
       *value = src->buffer_count;
       break;
-    case GST_PAD_QUERY_START:
+    case GST_QUERY_START:
       *value = src->segment_start;
       break;
-    case GST_PAD_QUERY_SEGMENT_END:
+    case GST_QUERY_SEGMENT_END:
       *value = src->segment_end;
       break;
     default:
@@ -355,7 +367,7 @@ gst_fakesrc_query (GstPad *pad, GstPadQueryType type,
   return TRUE;
 }
 
-GST_EVENT_MASK_FUNCTION (gst_fakesrc_get_event_mask,
+GST_PAD_EVENT_MASK_FUNCTION (gst_fakesrc_get_event_mask,
   { GST_EVENT_SEEK, GST_SEEK_FLAG_FLUSH },
   { GST_EVENT_SEEK_SEGMENT, GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SEGMENT_LOOP },
   { GST_EVENT_FLUSH, 0 }
@@ -502,11 +514,14 @@ gst_fakesrc_set_property (GObject *object, guint prop_id, const GValue *value, G
       break;
     case ARG_EOS:
       src->eos = g_value_get_boolean (value);
-      GST_INFO (0, "will EOS on next buffer");
+      GST_INFO ( "will EOS on next buffer");
       break;
     case ARG_SILENT:
       src->silent = g_value_get_boolean (value);
       break;
+    case ARG_SIGNAL_HANDOFFS:
+      src->signal_handoffs = g_value_get_boolean (value);
+      break;
     case ARG_DUMP:
       src->dump = g_value_get_boolean (value);
       break;
@@ -565,6 +580,9 @@ gst_fakesrc_get_property (GObject *object, guint prop_id, GValue *value, GParamS
     case ARG_SILENT:
       g_value_set_boolean (value, src->silent);
       break;
+    case ARG_SIGNAL_HANDOFFS:
+      g_value_set_boolean (value, src->signal_handoffs);
+      break;
     case ARG_DUMP:
       g_value_set_boolean (value, src->dump);
       break;
@@ -716,7 +734,7 @@ gst_fakesrc_create_buffer (GstFakeSrc *src)
   return buf;
 }
 
-static GstBuffer *
+static GstData *
 gst_fakesrc_get(GstPad *pad)
 {
   GstFakeSrc *src;
@@ -724,28 +742,28 @@ gst_fakesrc_get(GstPad *pad)
 
   g_return_val_if_fail (pad != NULL, NULL);
 
-  src = GST_FAKESRC (gst_pad_get_parent (pad));
+  src = GST_FAKESRC (GST_OBJECT_PARENT (pad));
 
   g_return_val_if_fail (GST_IS_FAKESRC (src), NULL);
 
   if (src->need_flush) {
     src->need_flush = FALSE;
-    return GST_BUFFER(gst_event_new (GST_EVENT_FLUSH));
+    return GST_DATA(gst_event_new (GST_EVENT_FLUSH));
   }
 
   if (src->buffer_count == src->segment_end) {
     if (src->segment_loop) {
-      return GST_BUFFER(gst_event_new (GST_EVENT_SEGMENT_DONE));
+      return GST_DATA(gst_event_new (GST_EVENT_SEGMENT_DONE));
     }
     else {
       gst_element_set_eos (GST_ELEMENT (src));
-      return GST_BUFFER(gst_event_new (GST_EVENT_EOS));
+      return GST_DATA(gst_event_new (GST_EVENT_EOS));
     }
   }
 
   if (src->rt_num_buffers == 0) {
     gst_element_set_eos (GST_ELEMENT (src));
-    return GST_BUFFER(gst_event_new (GST_EVENT_EOS));
+    return GST_DATA(gst_event_new (GST_EVENT_EOS));
   }
   else {
     if (src->rt_num_buffers > 0)
@@ -753,8 +771,8 @@ gst_fakesrc_get(GstPad *pad)
   }
 
   if (src->eos) {
-    GST_INFO (0, "fakesrc is setting eos on pad");
-    return GST_BUFFER(gst_event_new (GST_EVENT_EOS));
+    GST_INFO ( "fakesrc is setting eos on pad");
+    return GST_DATA(gst_event_new (GST_EVENT_EOS));
   }
 
   buf = gst_fakesrc_create_buffer (src);
@@ -763,18 +781,21 @@ gst_fakesrc_get(GstPad *pad)
   if (!src->silent) {
     g_free (src->last_message);
 
-    src->last_message = g_strdup_printf ("get      ******* (%s:%s)> (%d bytes, %llu) %p",
+    src->last_message = g_strdup_printf ("get      ******* (%s:%s)> (%d bytes, %"
+                                         G_GUINT64_FORMAT " ) %p",
                       GST_DEBUG_PAD_NAME (pad), GST_BUFFER_SIZE (buf), GST_BUFFER_TIMESTAMP (buf), buf);
 
     g_object_notify (G_OBJECT (src), "last_message");
   }
 
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, src, "pre handoff emit");
-  g_signal_emit (G_OBJECT (src), gst_fakesrc_signals[SIGNAL_HANDOFF], 0,
+  if (src->signal_handoffs) {
+    GST_LOG_OBJECT (src, "pre handoff emit");
+    g_signal_emit (G_OBJECT (src), gst_fakesrc_signals[SIGNAL_HANDOFF], 0,
                    buf, pad);
-  GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, src, "post handoff emit");
+    GST_LOG_OBJECT (src, "post handoff emit");
+  }
 
-  return buf;
+  return GST_DATA (buf);
 }
 
 /**
@@ -798,10 +819,10 @@ gst_fakesrc_loop(GstElement *element)
 
   while (pads) {
     GstPad *pad = GST_PAD (pads->data);
-    GstBuffer *buf;
+    GstData *data;
 
-    buf = gst_fakesrc_get (pad);
-    gst_pad_push (pad, buf);
+    data = gst_fakesrc_get (pad);
+    gst_pad_push (pad, data);
 
     if (src->eos) {
       return;
@@ -865,4 +886,3 @@ gst_fakesrc_factory_init (GstElementFactory *factory)
 
   return TRUE;
 }
-
