@@ -20,9 +20,15 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <gstfdsink.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
+#include "gstfdsink.h"
 #include <unistd.h>
 
+GST_DEBUG_CATEGORY (gst_fdsink_debug);
+#define GST_CAT_DEFAULT gst_fdsink_debug
 
 GstElementDetails gst_fdsink_details = {
   "Filedescriptor Sink",
@@ -55,7 +61,7 @@ static void 	gst_fdsink_set_property	(GObject *object, guint prop_id,
 static void 	gst_fdsink_get_property	(GObject *object, guint prop_id, 
 					 GValue *value, GParamSpec *pspec);
 
-static void 	gst_fdsink_chain	(GstPad *pad,GstBuffer *buf);
+static void 	gst_fdsink_chain	(GstPad *pad,GstData *_data);
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_fdsink_signals[LAST_SIGNAL] = { 0 };*/
@@ -109,8 +115,9 @@ gst_fdsink_init (GstFdSink *fdsink)
 }
 
 static void 
-gst_fdsink_chain (GstPad *pad, GstBuffer *buf) 
+gst_fdsink_chain (GstPad *pad, GstData *_data) 
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstFdSink *fdsink;
 
   g_return_if_fail (pad != NULL);
@@ -122,7 +129,7 @@ gst_fdsink_chain (GstPad *pad, GstBuffer *buf)
   g_return_if_fail (fdsink->fd >= 0);
   
   if (GST_BUFFER_DATA (buf)) {
-    GST_DEBUG (0,"writing %d bytes to file descriptor %d",GST_BUFFER_SIZE (buf), fdsink->fd);
+    GST_DEBUG ("writing %d bytes to file descriptor %d",GST_BUFFER_SIZE (buf), fdsink->fd);
     write (fdsink->fd, GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
   }
   
