@@ -27,10 +27,19 @@
 #include <unistd.h>
 #include <stdlib.h>
 
-#include <gstfdsrc.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
+#include "gstfdsrc.h"
 
-GstElementDetails gst_fdsrc_details = {
+#define DEFAULT_BLOCKSIZE	4096
+
+GST_DEBUG_CATEGORY (gst_fdsrc_debug);
+#define GST_CAT_DEFAULT gst_fdsrc_debug
+
+GstElementDetails gst_fdsrc_details = 
+{
   "Disk Source",
   "Source/File",
   "LGPL",
@@ -49,10 +58,8 @@ enum {
 
 enum {
   ARG_0,
-  ARG_LOCATION,
-  ARG_BYTESPERREAD,
-  ARG_OFFSET,
   ARG_FD,
+  ARG_BLOCKSIZE,
 };
 
 
@@ -64,7 +71,7 @@ static void		gst_fdsrc_set_property	(GObject *object, guint prop_id,
 static void		gst_fdsrc_get_property	(GObject *object, guint prop_id, 
 						 GValue *value, GParamSpec *pspec);
 
-static GstBuffer *	gst_fdsrc_get		(GstPad *pad);
+static GstData *	gst_fdsrc_get		(GstPad *pad);
 
 
 static GstElementClass *parent_class = NULL;
@@ -77,7 +84,8 @@ gst_fdsrc_get_type (void)
 
   if (!fdsrc_type) {
     static const GTypeInfo fdsrc_info = {
-      sizeof(GstFdSrcClass),      NULL,
+      sizeof(GstFdSrcClass),      
+      NULL,
       NULL,
       (GClassInitFunc)gst_fdsrc_class_init,
       NULL,
@@ -100,29 +108,26 @@ gst_fdsrc_class_init (GstFdSrcClass *klass)
 
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
 
-  gst_element_class_install_std_props (
-	  GST_ELEMENT_CLASS (klass),
-	  "location",     ARG_LOCATION,     G_PARAM_WRITABLE,
-	  "bytesperread", ARG_BYTESPERREAD, G_PARAM_READWRITE,
-	  "offset",       ARG_OFFSET,       G_PARAM_READABLE,
-	  NULL);
-
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FD,
     g_param_spec_int ("fd", "fd", "An open file descriptor to read from",
                       0, G_MAXINT, 0, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BLOCKSIZE,
+    g_param_spec_ulong ("blocksize", "Block size", "Size in bytes to read per buffer",
+                        1, G_MAXULONG, DEFAULT_BLOCKSIZE, G_PARAM_READWRITE));
 
   gobject_class->set_property = gst_fdsrc_set_property;
   gobject_class->get_property = gst_fdsrc_get_property;
 }
 
 static void gst_fdsrc_init(GstFdSrc *fdsrc) {
-  fdsrc->srcpad = gst_pad_new("src",GST_PAD_SRC);
-  gst_pad_set_get_function(fdsrc->srcpad,gst_fdsrc_get);
-  gst_element_add_pad(GST_ELEMENT(fdsrc),fdsrc->srcpad);
+  fdsrc->srcpad = gst_pad_new ("src", GST_PAD_SRC);
+  
+  gst_pad_set_get_function (fdsrc->srcpad, gst_fdsrc_get);
+  gst_element_add_pad (GST_ELEMENT (fdsrc), fdsrc->srcpad);
 
   fdsrc->fd = 0;
   fdsrc->curoffset = 0;
-  fdsrc->bytes_per_read = 4096;
+  fdsrc->blocksize = DEFAULT_BLOCKSIZE;
   fdsrc->seq = 0;
 }
 
@@ -131,7 +136,6 @@ static void
 gst_fdsrc_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec) 
 {
   GstFdSrc *src;
-  int fd;
 
   /* it's not null if we got it, but it might not be ours */
   g_return_if_fail (GST_IS_FDSRC (object));
@@ -139,25 +143,11 @@ gst_fdsrc_set_property (GObject *object, guint prop_id, const GValue *value, GPa
   src = GST_FDSRC (object);
 
   switch (prop_id) {
-    case ARG_LOCATION:
-      /* the element must not be playing in order to do this */
-      g_return_if_fail (GST_STATE (src) < GST_STATE_PLAYING);
-
-      /* if we get a NULL, consider it to be a fd of 0 */
-      if (g_value_get_string (value) == NULL) {
-        gst_element_set_state (GST_ELEMENT (object), GST_STATE_NULL);
-        src->fd = 0;
-      /* otherwise set the new filename */
-      } else {
-        if (sscanf (g_value_get_string (value), "%d", &fd))
-          src->fd = fd;
-      }
-      break;
     case ARG_FD:
       src->fd = g_value_get_int (value);
       break;
-    case ARG_BYTESPERREAD:
-      src->bytes_per_read = g_value_get_int (value);
+    case ARG_BLOCKSIZE:
+      src->blocksize = g_value_get_ulong (value);
       break;
     default:
       break;
@@ -175,11 +165,8 @@ gst_fdsrc_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
   src = GST_FDSRC (object);
 
   switch (prop_id) {
-    case ARG_BYTESPERREAD:
-      g_value_set_int (value, src->bytes_per_read);
-      break;
-    case ARG_OFFSET:
-      g_value_set_int64 (value, src->curoffset);
+    case ARG_BLOCKSIZE:
+      g_value_set_ulong (value, src->blocksize);
       break;
     case ARG_FD:
       g_value_set_int (value, src->fd);
@@ -190,37 +177,38 @@ gst_fdsrc_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
   }
 }
 
-static GstBuffer *
+static GstData *
 gst_fdsrc_get(GstPad *pad)
 {
   GstFdSrc *src;
   GstBuffer *buf;
   glong readbytes;
 
-  g_return_val_if_fail (pad != NULL, NULL);
-  src = GST_FDSRC(gst_pad_get_parent (pad));
+  src = GST_FDSRC (gst_pad_get_parent (pad));
 
   /* create the buffer */
-  /* FIXME: should eventually use a bufferpool for this*/
-  buf = gst_buffer_new ();
-  g_return_val_if_fail (buf, NULL);
-
-  /* allocate the space for the buffer data */
-  GST_BUFFER_DATA(buf) = g_malloc(src->bytes_per_read);
-  g_return_val_if_fail(GST_BUFFER_DATA(buf) != NULL, NULL);
+  buf = gst_buffer_new_and_alloc (src->blocksize);
 
   /* read it in from the file */
-  readbytes = read(src->fd,GST_BUFFER_DATA(buf),src->bytes_per_read);
+  readbytes = read (src->fd, GST_BUFFER_DATA (buf), src->blocksize);
+
   /* if nothing was read, we're in eos */
   if (readbytes == 0) {
     gst_element_set_eos (GST_ELEMENT (src));
-    return GST_BUFFER (gst_event_new (GST_EVENT_EOS));
+    return GST_DATA (gst_event_new (GST_EVENT_EOS));
   }
 
-  GST_BUFFER_OFFSET(buf) = src->curoffset;
-  GST_BUFFER_SIZE(buf) = readbytes;
+  if (readbytes == -1) {
+    g_error ("Error reading from file descriptor. Ending stream.\n");
+    gst_element_set_eos (GST_ELEMENT (src));
+    return GST_DATA (gst_event_new (GST_EVENT_EOS));
+  }
+  
+  GST_BUFFER_OFFSET (buf) = src->curoffset;
+  GST_BUFFER_SIZE (buf) = readbytes;
+  GST_BUFFER_TIMESTAMP (buf) = GST_CLOCK_TIME_NONE;
   src->curoffset += readbytes;
 
   /* we're done, return the buffer */
-  return buf;
+  return GST_DATA (buf);
 }
