@@ -20,8 +20,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <gst/gst.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
+#include <gst/gst.h>
 #include "gstfilesrc.h"
 
 #include <stdio.h>
@@ -69,6 +72,9 @@
  */
 
 
+GST_DEBUG_CATEGORY (gst_filesrc_debug);
+#define GST_CAT_DEFAULT gst_filesrc_debug
+
 GstElementDetails gst_filesrc_details = {
   "File Source",
   "Source/File",
@@ -136,7 +142,7 @@ static void		gst_filesrc_set_property	(GObject *object, guint prop_id,
 static void		gst_filesrc_get_property	(GObject *object, guint prop_id, 
 							 GValue *value, GParamSpec *pspec);
 
-static GstBuffer *	gst_filesrc_get			(GstPad *pad);
+static GstData *	gst_filesrc_get			(GstPad *pad);
 static gboolean 	gst_filesrc_srcpad_event 	(GstPad *pad, GstEvent *event);
 static gboolean 	gst_filesrc_srcpad_query 	(GstPad *pad, GstQueryType type,
 		         				 GstFormat *format, gint64 *value);
@@ -299,7 +305,7 @@ gst_filesrc_set_property (GObject *object, guint prop_id, const GValue *value, G
         src->mapsize = g_value_get_ulong (value);
         g_object_notify (G_OBJECT (src), "mmapsize");
       } else {
-        GST_INFO (0, "invalid mapsize, must a multiple of pagesize, which is %d", 
+        GST_INFO ( "invalid mapsize, must a multiple of pagesize, which is %d", 
 	          src->pagesize);
       }
       break;
@@ -367,8 +373,11 @@ gst_filesrc_free_parent_mmap (GstBuffer *buf)
 #endif
   /* now unmap the memory */
   munmap (GST_BUFFER_DATA (buf), GST_BUFFER_MAXSIZE (buf));
-  GST_DEBUG (0, "unmapped region %08llx+%08llx at %p", 
-		  GST_BUFFER_OFFSET (buf), GST_BUFFER_MAXSIZE (buf), 
+  /* cast to unsigned long, since there's no gportable way to print
+   * guint64 as hex */
+  GST_DEBUG ( "unmapped region %08lx+%08lx at %p", 
+		  (unsigned long) GST_BUFFER_OFFSET (buf),
+		  (unsigned long) GST_BUFFER_MAXSIZE (buf), 
 		  GST_BUFFER_DATA (buf));
 
   GST_BUFFER_DATA (buf) = NULL;
@@ -385,7 +394,7 @@ gst_filesrc_map_region (GstFileSrc *src, off_t offset, size_t size)
 
   g_return_val_if_fail (offset >= 0, NULL);
 
-  fs_print  ("mapping region %08llx+%08x from file into memory\n",offset,size);
+  fs_print  ("mapping region %08llx+%08lx from file into memory\n",offset,(unsigned long)size);
   mmapregion = mmap (NULL, size, PROT_READ, MAP_SHARED, src->fd, offset);
 
   if (mmapregion == NULL) {
@@ -393,12 +402,12 @@ gst_filesrc_map_region (GstFileSrc *src, off_t offset, size_t size)
     return NULL;
   }
   else if (mmapregion == MAP_FAILED) {
-    gst_element_error (GST_ELEMENT (src), "mmap (0x%x, %d, 0x%llx) : %s",
- 	     size, src->fd, offset, strerror (errno));
+    GST_DEBUG ("mmap (0x%08lx, %d, 0x%llx) : %s",
+ 	     (unsigned long)size, src->fd, offset, strerror (errno));
     return NULL;
   }
-  GST_DEBUG (0, "mapped region %08llx+%08x from file into memory at %p", 
-		  offset, size, mmapregion);
+  GST_DEBUG ( "mapped region %08lx+%08lx from file into memory at %p", 
+		  (unsigned long)offset, (unsigned long)size, mmapregion);
 
   /* time to allocate a new mapbuf */
   buf = gst_buffer_new ();
@@ -447,6 +456,7 @@ gst_filesrc_map_small_region (GstFileSrc *src, off_t offset, size_t size)
       return NULL;
 
     ret = gst_buffer_create_sub (map, offset - mapbase, size);
+    GST_BUFFER_OFFSET (ret) = GST_BUFFER_OFFSET (map) + offset - mapbase;
 
     gst_buffer_unref (map);
 
@@ -476,54 +486,26 @@ gst_filesrc_search_region_match (gpointer a, gpointer b)
 }
 
 /**
- * gst_filesrc_get:
+ * gst_filesrc_get_mmap:
  * @pad: #GstPad to push a buffer from
  *
  * Push a new buffer from the filesrc at the current offset.
  */
 static GstBuffer *
-gst_filesrc_get (GstPad *pad)
+gst_filesrc_get_mmap (GstFileSrc *src)
 {
-  GstFileSrc *src;
   GstBuffer *buf = NULL, *map;
-  size_t readsize;
+  size_t readsize, mapsize;
   off_t readend,mapstart,mapend;
   GstFileSrcRegion region;
   int i;
 
-  g_return_val_if_fail (pad != NULL, NULL);
-  src = GST_FILESRC (gst_pad_get_parent (pad));
-  g_return_val_if_fail (GST_FLAG_IS_SET (src, GST_FILESRC_OPEN), NULL);
-
-  /* check for seek */
-  if (src->seek_happened) {
-    GstEvent *event;
-
-    src->seek_happened = FALSE;
-    GST_DEBUG (GST_CAT_EVENT, "filesrc sending discont");
-    event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, src->curoffset, NULL);
-    src->need_flush = FALSE;
-    return GST_BUFFER (event);
-  }
-  /* check for flush */
-  if (src->need_flush) {
-    src->need_flush = FALSE;
-    GST_DEBUG (GST_CAT_EVENT, "filesrc sending flush");
-    return GST_BUFFER (gst_event_new_flush ());
-  }
-
-  /* check for EOF */
-  if (src->curoffset == src->filelen) {
-    GST_DEBUG (0, "filesrc eos %lld %lld", src->curoffset, src->filelen);
-    gst_element_set_eos (GST_ELEMENT (src));
-    return GST_BUFFER (gst_event_new (GST_EVENT_EOS));
-  }
-
   /* calculate end pointers so we don't have to do so repeatedly later */
   readsize = src->block_size;
   readend = src->curoffset + src->block_size;		/* note this is the byte *after* the read */
-  mapstart = GST_BUFFER_OFFSET(src->mapbuf);
-  mapend = mapstart + GST_BUFFER_SIZE(src->mapbuf);	/* note this is the byte *after* the map */
+  mapstart = GST_BUFFER_OFFSET (src->mapbuf);
+  mapsize = GST_BUFFER_SIZE (src->mapbuf);
+  mapend = mapstart + mapsize;			/* note this is the byte *after* the map */
 
   /* check to see if we're going to overflow the end of the file */
   if (readend > src->filelen) {
@@ -531,8 +513,9 @@ gst_filesrc_get (GstPad *pad)
     readend = src->curoffset + readsize;
   }
 
-  GST_DEBUG (0, "attempting to read %08x, %08llx, %08llx, %08llx", 
-		  readsize, readend, mapstart, mapend);
+  GST_LOG ("attempting to read %08lx, %08lx, %08lx, %08lx", 
+	   (unsigned long)readsize, (unsigned long)readend,
+	   (unsigned long)mapstart, (unsigned long)mapend);
 
   /* if the start is past the mapstart */
   if (src->curoffset >= mapstart) {
@@ -540,14 +523,15 @@ gst_filesrc_get (GstPad *pad)
     /* ('cause by definition if readend is in the buffer, so's readstart) */
     if (readend <= mapend) {
       fs_print ("read buf %llu+%d lives in current mapbuf %lld+%d, creating subbuffer of mapbuf\n",
-             src->curoffset, readsize, mapstart, GST_BUFFER_SIZE(src->mapbuf));
-      buf = gst_buffer_create_sub (src->mapbuf, src->curoffset - GST_BUFFER_OFFSET(src->mapbuf),
+             src->curoffset, readsize, mapstart, mapsize);
+      buf = gst_buffer_create_sub (src->mapbuf, src->curoffset - mapstart,
                                    readsize);
+      GST_BUFFER_OFFSET (buf) = src->curoffset;
 
     /* if the start actually is within the current mmap region, map an overlap buffer */
     } else if (src->curoffset < mapend) {
       fs_print ("read buf %llu+%d starts in mapbuf %d+%d but ends outside, creating new mmap\n",
-             src->curoffset,readsize,GST_BUFFER_OFFSET(src->mapbuf),GST_BUFFER_SIZE(src->mapbuf));
+             src->curoffset, readsize, mapstart, mapsize);
       buf = gst_filesrc_map_small_region (src, src->curoffset, readsize);
       if (buf == NULL)
         return NULL;
@@ -562,7 +546,7 @@ gst_filesrc_get (GstPad *pad)
     /* or the read buffer fully contains the current mmap region    */
     /* either way, it's really not relevant, we just create a new region anyway*/
     fs_print ("read buf %llu+%d starts before mapbuf %d+%d, but overlaps it\n",
-             src->curoffset,readsize,GST_BUFFER_OFFSET(src->mapbuf),GST_BUFFER_SIZE(src->mapbuf));
+             src->curoffset,readsize, mapstart, mapsize);
     buf = gst_filesrc_map_small_region (src, src->curoffset, readsize);
     if (buf == NULL)
       return NULL;
@@ -582,6 +566,7 @@ gst_filesrc_get (GstPad *pad)
     if (map != NULL) {
       fs_print ("found mapbuf at %d+%d, creating subbuffer\n",GST_BUFFER_OFFSET(map),GST_BUFFER_SIZE(map));
       buf = gst_buffer_create_sub (map, src->curoffset - GST_BUFFER_OFFSET(map), readsize);
+      GST_BUFFER_OFFSET (buf) = src->curoffset;
 
     /* otherwise we need to create something out of thin air */
     } else {
@@ -616,6 +601,7 @@ gst_filesrc_get (GstPad *pad)
 
         /* subbuffer it */
         buf = gst_buffer_create_sub (src->mapbuf, src->curoffset - nextmap, readsize);
+        GST_BUFFER_OFFSET (buf) = mapstart + src->curoffset - nextmap;
       }
     }
   }
@@ -633,13 +619,85 @@ gst_filesrc_get (GstPad *pad)
   return buf;
 }
 
+static GstBuffer *
+gst_filesrc_get_read (GstFileSrc *src)
+{
+  GstBuffer *buf = NULL;
+  size_t readsize;
+  int ret;
+
+  readsize = src->block_size;
+  if (src->curoffset + readsize > src->filelen) {
+    readsize = src->filelen - src->curoffset;
+  }
+
+  buf = gst_buffer_new_and_alloc (readsize);
+  g_return_val_if_fail (buf != NULL, NULL);
+
+  ret = read (src->fd, GST_BUFFER_DATA (buf), readsize);
+  if (ret < 0){
+    gst_element_error (GST_ELEMENT (src), "reading file (%s)",
+        strerror (errno), NULL);
+    return NULL;
+  }
+  if (ret < readsize) {
+    gst_element_error (GST_ELEMENT (src), "unexpected end of file", NULL);
+    return NULL;
+  }
+
+  src->curoffset += readsize;
+
+  return buf;
+}
+
+static GstData *
+gst_filesrc_get (GstPad *pad)
+{
+  GstFileSrc *src;
+
+  g_return_val_if_fail (pad != NULL, NULL);
+  src = GST_FILESRC (gst_pad_get_parent (pad));
+  g_return_val_if_fail (GST_FLAG_IS_SET (src, GST_FILESRC_OPEN), NULL);
+
+  /* check for seek */
+  if (src->seek_happened) {
+    GstEvent *event;
+
+    src->seek_happened = FALSE;
+    GST_DEBUG ("filesrc sending discont");
+    event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, src->curoffset, NULL);
+    src->need_flush = FALSE;
+    return GST_DATA (event);
+  }
+  /* check for flush */
+  if (src->need_flush) {
+    src->need_flush = FALSE;
+    GST_DEBUG ("filesrc sending flush");
+    return GST_DATA (gst_event_new_flush ());
+  }
+
+  /* check for EOF */
+  if (src->curoffset == src->filelen) {
+    GST_DEBUG ("filesrc eos %" G_GINT64_FORMAT" %" G_GINT64_FORMAT,
+               src->curoffset, src->filelen);
+    gst_element_set_eos (GST_ELEMENT (src));
+    return GST_DATA (gst_event_new (GST_EVENT_EOS));
+  }
+
+  if (src->using_mmap){
+    return GST_DATA (gst_filesrc_get_mmap (src));
+  }else{
+    return GST_DATA (gst_filesrc_get_read (src));
+  }
+}
+
 /* open the file and mmap it, necessary to go to READY state */
 static gboolean 
 gst_filesrc_open_file (GstFileSrc *src)
 {
   g_return_val_if_fail (!GST_FLAG_IS_SET (src ,GST_FILESRC_OPEN), FALSE);
 
-  GST_DEBUG(0, "opening file %s",src->filename);
+  GST_DEBUG ( "opening file %s",src->filename);
 
   /* open the file */
   src->fd = open (src->filename, O_RDONLY);
@@ -661,13 +719,15 @@ gst_filesrc_open_file (GstFileSrc *src)
     }
 		
     /* find the file length */
-    src->filelen = lseek (src->fd, 0, SEEK_END);
-    lseek (src->fd, 0, SEEK_SET);
+    src->filelen = stat_results.st_size;
 
     /* allocate the first mmap'd region */
     src->mapbuf = gst_filesrc_map_region (src, 0, src->mapsize);
-    if (src->mapbuf == NULL)
-      return FALSE;
+    if (src->mapbuf == NULL) {
+      src->using_mmap = FALSE;
+    }else{
+      src->using_mmap = TRUE;
+    }
 
     src->curoffset = 0;
 
@@ -769,12 +829,12 @@ gst_filesrc_srcpad_event (GstPad *pad, GstEvent *event)
 {
   GstFileSrc *src = GST_FILESRC (GST_PAD_PARENT (pad));
 
-  GST_DEBUG(0, "event %d", GST_EVENT_TYPE (event));
+  GST_DEBUG ( "event %d", GST_EVENT_TYPE (event));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
     {
-      guint64 offset;
+      gint64 offset;
 
       if (GST_EVENT_SEEK_FORMAT (event) != GST_FORMAT_BYTES) {
 	goto error;
@@ -787,19 +847,19 @@ gst_filesrc_srcpad_event (GstPad *pad, GstEvent *event)
           if (offset > src->filelen) 
 	    goto error;
           src->curoffset = offset;
-          GST_DEBUG(0, "seek set pending to %lld", src->curoffset);
+          GST_DEBUG ( "seek set pending to %" G_GINT64_FORMAT, src->curoffset);
 	  break;
         case GST_SEEK_METHOD_CUR:
           if (offset + src->curoffset > src->filelen) 
 	    goto error;
           src->curoffset += offset;
-          GST_DEBUG(0, "seek cur pending to %lld", src->curoffset);
+          GST_DEBUG ( "seek cur pending to %" G_GINT64_FORMAT, src->curoffset);
 	  break;
         case GST_SEEK_METHOD_END:
           if (ABS (offset) > src->filelen) 
 	    goto error;
           src->curoffset = src->filelen - ABS (offset);
-          GST_DEBUG(0, "seek end pending to %lld", src->curoffset);
+          GST_DEBUG ( "seek end pending to %" G_GINT64_FORMAT, src->curoffset);
 	  break;
 	default:
           goto error;
