@@ -21,13 +21,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* #define GST_DEBUG_ENABLED */
 #include "gst_private.h"
 
 #include "gstevent.h"
 #include "gstbin.h"
 #include "gstxml.h"
-#include "gstlog.h"
+#include "gstinfo.h"
 
 #include "gstscheduler.h"
 #include "gstindex.h"
@@ -44,21 +43,31 @@ GstElementDetails gst_bin_details = {
 
 GType _gst_bin_type = 0;
 
+static gboolean _gst_boolean_did_something_accumulator (GSignalInvocationHint *ihint,
+    GValue *return_accu, const GValue *handler_return, gpointer dummy);
+
 static void 			gst_bin_dispose 		(GObject * object);
 
 static GstElementStateReturn	gst_bin_change_state		(GstElement *element);
 static GstElementStateReturn	gst_bin_change_state_norecurse	(GstBin *bin);
 
+#ifndef GST_DISABLE_INDEX
 static void 			gst_bin_set_index 		(GstElement *element, GstIndex *index);
+#endif
 
 static void 			gst_bin_add_func 		(GstBin *bin, GstElement *element);
 static void 			gst_bin_remove_func 		(GstBin *bin, GstElement *element);
+static void			gst_bin_child_state_change_func (GstBin *bin, GstElementState oldstate, 
+                                				 GstElementState newstate, GstElement *child);
+
+static GstClock* 		gst_bin_get_clock_func 		(GstElement *element);
+static void	 		gst_bin_set_clock_func 		(GstElement *element, GstClock *clock);
 
 static gboolean 		gst_bin_iterate_func 		(GstBin *bin);
 
 #ifndef GST_DISABLE_LOADSAVE
-static xmlNodePtr 		gst_bin_save_thyself 		(GstObject *object, xmlNodePtr parent);
-static void 			gst_bin_restore_thyself 	(GstObject *object, xmlNodePtr self);
+static xmlNodePtr 		gst_bin_save_thyself 		(GstObject * object, xmlNodePtr parent);
+static void 			gst_bin_restore_thyself 	(GstObject * object, xmlNodePtr self);
 #endif
 
 /* Bin signals and args */
@@ -66,12 +75,13 @@ enum
 {
   ELEMENT_ADDED,
   ELEMENT_REMOVED,
+  ITERATE,
   LAST_SIGNAL
 };
 
 enum
 {
-  ARG_0,
+  ARG_0
   /* FILL ME */
 };
 
@@ -124,6 +134,11 @@ gst_bin_class_init (GstBinClass * klass)
     g_signal_new ("element_removed", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GstBinClass, element_removed), NULL, NULL,
 		  gst_marshal_VOID__POINTER, G_TYPE_NONE, 1, G_TYPE_POINTER);
+  gst_bin_signals[ITERATE] =
+    g_signal_new ("iterate", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (GstBinClass, iterate),
+                  _gst_boolean_did_something_accumulator, NULL,
+		  gst_marshal_BOOLEAN__VOID, G_TYPE_BOOLEAN, 0);
 
   gobject_class->dispose 		= GST_DEBUG_FUNCPTR (gst_bin_dispose);
 
@@ -133,13 +148,31 @@ gst_bin_class_init (GstBinClass * klass)
 #endif
 
   gstelement_class->change_state 	= GST_DEBUG_FUNCPTR (gst_bin_change_state);
+#ifndef GST_DISABLE_INDEX
   gstelement_class->set_index	 	= GST_DEBUG_FUNCPTR (gst_bin_set_index);
+#endif
 
   klass->add_element 			= GST_DEBUG_FUNCPTR (gst_bin_add_func);
   klass->remove_element 		= GST_DEBUG_FUNCPTR (gst_bin_remove_func);
+  klass->child_state_change		= GST_DEBUG_FUNCPTR (gst_bin_child_state_change_func);
   klass->iterate 			= GST_DEBUG_FUNCPTR (gst_bin_iterate_func);
 }
 
+static gboolean
+_gst_boolean_did_something_accumulator (GSignalInvocationHint *ihint,
+    GValue *return_accu, const GValue *handler_return, gpointer dummy)
+{
+  gboolean did_something;
+
+  did_something = g_value_get_boolean (handler_return);
+  if (did_something) {
+    g_value_set_boolean (return_accu, TRUE);
+  }
+
+  /* always continue emission */
+  return TRUE;
+}
+
 static void
 gst_bin_init (GstBin * bin)
 {
@@ -148,11 +181,6 @@ gst_bin_init (GstBin * bin)
 
   bin->numchildren = 0;
   bin->children = NULL;
-  
-  bin->pre_iterate_func = NULL;
-  bin->post_iterate_func = NULL;
-  bin->pre_iterate_data = NULL;
-  bin->post_iterate_data = NULL;
 }
 
 /**
@@ -169,6 +197,22 @@ gst_bin_new (const gchar * name)
   return gst_element_factory_make ("bin", name);
 }
 
+static GstClock*
+gst_bin_get_clock_func (GstElement *element)
+{
+  if (GST_ELEMENT_SCHED (element)) 
+    return gst_scheduler_get_clock (GST_ELEMENT_SCHED (element));
+
+  return NULL;
+}
+
+static void
+gst_bin_set_clock_func (GstElement *element, GstClock *clock)
+{
+  if (GST_ELEMENT_SCHED (element)) 
+    gst_scheduler_use_clock (GST_ELEMENT_SCHED (element), clock);
+}
+
 /**
  * gst_bin_get_clock:
  * @bin: a #GstBin to get the clock of
@@ -183,10 +227,7 @@ gst_bin_get_clock (GstBin *bin)
   g_return_val_if_fail (bin != NULL, NULL);
   g_return_val_if_fail (GST_IS_BIN (bin), NULL);
 
-  if (GST_ELEMENT_SCHED (bin)) 
-    return gst_scheduler_get_clock (GST_ELEMENT_SCHED (bin));
-
-  return NULL;
+  return gst_bin_get_clock_func (GST_ELEMENT (bin));
 }
 
 /**
@@ -200,11 +241,9 @@ gst_bin_get_clock (GstBin *bin)
 void
 gst_bin_use_clock (GstBin *bin, GstClock *clock)
 {
-  g_return_if_fail (bin != NULL);
   g_return_if_fail (GST_IS_BIN (bin));
 
-  if (GST_ELEMENT_SCHED (bin)) 
-    gst_scheduler_use_clock (GST_ELEMENT_SCHED (bin), clock);
+  gst_bin_set_clock_func (GST_ELEMENT (bin), clock);
 }
 
 /**
@@ -216,56 +255,44 @@ gst_bin_use_clock (GstBin *bin, GstClock *clock)
 void
 gst_bin_auto_clock (GstBin *bin)
 {
-  g_return_if_fail (bin != NULL);
   g_return_if_fail (GST_IS_BIN (bin));
 
   if (GST_ELEMENT_SCHED (bin)) 
     gst_scheduler_auto_clock (GST_ELEMENT_SCHED (bin));
 }
 
+#ifndef GST_DISABLE_INDEX
 static void
 gst_bin_set_index (GstElement *element, GstIndex *index)
 {
-  GList *children;
   GstBin *bin = GST_BIN (element);
+  
+  g_return_if_fail (GST_IS_BIN (bin));
 
-  children = bin->children;
-  while (children) {
-    GstElement *child = GST_ELEMENT (children->data);
-    children = g_list_next (children);
-
-    gst_element_set_index (child, index);
-  }
+  g_list_foreach (bin->children, (GFunc) gst_element_set_index, index);
 }
+#endif
 
 static void
 gst_bin_set_element_sched (GstElement *element, GstScheduler *sched)
 {
-  GST_INFO (GST_CAT_SCHEDULING, "setting element \"%s\" sched to %p", GST_ELEMENT_NAME (element),
+  GST_CAT_INFO (GST_CAT_SCHEDULING, "setting element \"%s\" sched to %p", GST_ELEMENT_NAME (element),
 	    sched);
 
   /* if it's actually a Bin */
   if (GST_IS_BIN (element)) {
-    GList *children;
-
     if (GST_FLAG_IS_SET (element, GST_BIN_FLAG_MANAGER)) {
-      GST_INFO_ELEMENT (GST_CAT_PARENTAGE, element, "child is already a manager, not resetting");
+      GST_CAT_INFO (GST_CAT_PARENTAGE, "[%s]: child is already a manager, not resetting", GST_ELEMENT_NAME (element));
       if (GST_ELEMENT_SCHED (element))
         gst_scheduler_add_scheduler (sched, GST_ELEMENT_SCHED (element));
       return;
     }
 
-    GST_INFO_ELEMENT (GST_CAT_PARENTAGE, element, "setting children's schedule to parent's");
+    GST_CAT_INFO (GST_CAT_PARENTAGE, "[%s]: setting children's schedule to parent's", GST_ELEMENT_NAME (element));
     gst_scheduler_add_element (sched, element);
 
     /* set the children's schedule */
-    children = GST_BIN (element)->children;
-    while (children) {
-      GstElement *child = GST_ELEMENT (children->data);
-      children = g_list_next (children);
-
-      gst_bin_set_element_sched (child, sched);
-    }
+    g_list_foreach (GST_BIN (element)->children, (GFunc) gst_bin_set_element_sched, sched);
   }
   /* otherwise, if it's just a regular old element */
   else {
@@ -288,7 +315,7 @@ gst_bin_set_element_sched (GstElement *element, GstScheduler *sched)
       /* if the peer element exists and is a candidate */
       if (GST_PAD_PEER (pad)) {
         if (gst_pad_get_scheduler (GST_PAD_PEER (pad)) == sched) {
-          GST_INFO (GST_CAT_SCHEDULING,
+          GST_CAT_INFO (GST_CAT_SCHEDULING,
 		    "peer is in same scheduler, telling scheduler");
 
           if (GST_PAD_IS_SRC (pad))
@@ -305,37 +332,28 @@ gst_bin_set_element_sched (GstElement *element, GstScheduler *sched)
 static void
 gst_bin_unset_element_sched (GstElement *element, GstScheduler *sched)
 {
-  GList *children;
-  GstElement *child;
-
   if (GST_ELEMENT_SCHED (element) == NULL) {
-    GST_INFO (GST_CAT_SCHEDULING, "element \"%s\" has no scheduler",
+    GST_CAT_INFO (GST_CAT_SCHEDULING, "element \"%s\" has no scheduler",
 	      GST_ELEMENT_NAME (element));
     return;
   }
 
-  GST_INFO (GST_CAT_SCHEDULING, "removing element \"%s\" from its sched %p",
+  GST_CAT_INFO (GST_CAT_SCHEDULING, "removing element \"%s\" from its sched %p",
 	    GST_ELEMENT_NAME (element), GST_ELEMENT_SCHED (element));
 
   /* if it's actually a Bin */
   if (GST_IS_BIN (element)) {
 
     if (GST_FLAG_IS_SET (element, GST_BIN_FLAG_MANAGER)) {
-      GST_INFO_ELEMENT (GST_CAT_PARENTAGE, element,
-			"child is already a manager, not unsetting sched");
+      GST_CAT_INFO (GST_CAT_PARENTAGE, "[%s]: child is already a manager, not unsetting sched",
+                    GST_ELEMENT_NAME (element));
       if (sched) {
         gst_scheduler_remove_scheduler (sched, GST_ELEMENT_SCHED (element));
       }
       return;
     }
     /* for each child, remove them from their schedule */
-    children = GST_BIN (element)->children;
-    while (children) {
-      child = GST_ELEMENT (children->data);
-      children = g_list_next (children);
-
-      gst_bin_unset_element_sched (child, sched);
-    }
+    g_list_foreach (GST_BIN (element)->children, (GFunc) gst_bin_unset_element_sched, sched);
 
     gst_scheduler_remove_element (GST_ELEMENT_SCHED (element), element);
   }
@@ -357,14 +375,13 @@ gst_bin_unset_element_sched (GstElement *element, GstScheduler *sched)
 
       /* if the peer element exists and is a candidate */
       if (GST_PAD_PEER (pad)) {
-        if (sched && gst_pad_get_scheduler (GST_PAD_PEER (pad)) == sched) {
-          GST_INFO (GST_CAT_SCHEDULING, "peer is in same scheduler, telling scheduler");
+        if (gst_pad_get_scheduler (GST_PAD_PEER (pad)) == sched) {
+          GST_CAT_INFO (GST_CAT_SCHEDULING, "peer is in same scheduler, telling scheduler");
 
-          if (GST_PAD_IS_SRC (pad)){
+          if (GST_PAD_IS_SRC (pad))
             gst_scheduler_pad_unlink (sched, pad, GST_PAD_PEER (pad));
-	  }else{
+          else
             gst_scheduler_pad_unlink (sched, GST_PAD_PEER (pad), pad);
-	  }
         }
       }
     }
@@ -386,8 +403,6 @@ gst_bin_add_many (GstBin *bin, GstElement *element_1, ...)
 {
   va_list args;
 
-  g_return_if_fail (bin != NULL);
-  g_return_if_fail (element_1 != NULL);
   g_return_if_fail (GST_IS_BIN (bin));
   g_return_if_fail (GST_IS_ELEMENT (element_1));
 
@@ -409,6 +424,9 @@ gst_bin_add_func (GstBin *bin, GstElement *element)
   GstElementState state;
   GstScheduler *sched;
 
+  /* the element must not already have a parent */
+  g_return_if_fail (GST_ELEMENT_PARENT (element) == NULL);
+
   /* then check to see if the element's name is already taken in the bin */
   if (gst_object_check_uniqueness (bin->children, 
 	                           GST_ELEMENT_NAME (element)) == FALSE)
@@ -437,7 +455,8 @@ gst_bin_add_func (GstBin *bin, GstElement *element)
     gst_bin_set_element_sched (element, sched);
   }
 
-  GST_INFO_ELEMENT (GST_CAT_PARENTAGE, bin, "added child \"%s\"", GST_ELEMENT_NAME (element));
+  GST_CAT_INFO (GST_CAT_PARENTAGE, "[%s]: added child \"%s\"", 
+                GST_ELEMENT_NAME (bin), GST_ELEMENT_NAME (element));
 
   g_signal_emit (G_OBJECT (bin), gst_bin_signals[ELEMENT_ADDED], 0, element);
 }
@@ -455,20 +474,12 @@ gst_bin_add (GstBin *bin, GstElement *element)
 {
   GstBinClass *bclass;
   
-  g_return_if_fail (bin != NULL);
   g_return_if_fail (GST_IS_BIN (bin));
-  g_return_if_fail (element != NULL);
   g_return_if_fail (GST_IS_ELEMENT (element));
 
-  GST_DEBUG (GST_CAT_PARENTAGE, "adding element \"%s\" to bin \"%s\"",
+  GST_CAT_DEBUG (GST_CAT_PARENTAGE, "adding element \"%s\" to bin \"%s\"",
 	     GST_ELEMENT_NAME (element), GST_ELEMENT_NAME (bin));
 
-  /* the element must not already have a parent */
-  g_return_if_fail (GST_ELEMENT_PARENT (element) == NULL);
-
-  /* must be not be in PLAYING state in order to modify bin */
-  g_return_if_fail (GST_STATE (bin) != GST_STATE_PLAYING);
-
   bclass = GST_BIN_GET_CLASS (bin);
 
   if (bclass->add_element) {
@@ -507,7 +518,8 @@ gst_bin_remove_func (GstBin *bin, GstElement *element)
   while (state >>= 1) state_idx++;
   bin->child_states[state_idx]--;
 
-  GST_INFO_ELEMENT (GST_CAT_PARENTAGE, bin, "removed child %s", GST_ELEMENT_NAME (element));
+  GST_CAT_INFO (GST_CAT_PARENTAGE, "[%s]: removed child %s", 
+                GST_ELEMENT_NAME (bin), GST_ELEMENT_NAME (element));
 
   /* ref as we're going to emit a signal */
   gst_object_ref (GST_OBJECT (element));
@@ -540,17 +552,12 @@ gst_bin_remove (GstBin *bin, GstElement *element)
 {
   GstBinClass *bclass;
 
-  GST_DEBUG_ELEMENT (GST_CAT_PARENTAGE, bin, "trying to remove child %s", GST_ELEMENT_NAME (element));
+  GST_CAT_DEBUG (GST_CAT_PARENTAGE, "[%s]: trying to remove child %s", GST_ELEMENT_NAME (bin), GST_ELEMENT_NAME (element));
 
-  g_return_if_fail (bin != NULL);
   g_return_if_fail (GST_IS_BIN (bin));
-  g_return_if_fail (element != NULL);
   g_return_if_fail (GST_IS_ELEMENT (element));
   g_return_if_fail (bin->children != NULL);
 
-  /* must not be in PLAYING state in order to modify bin */
-  g_return_if_fail (GST_STATE (bin) != GST_STATE_PLAYING);
-
   bclass = GST_BIN_GET_CLASS (bin);
 
   if (bclass->remove_element) {
@@ -561,6 +568,33 @@ gst_bin_remove (GstBin *bin, GstElement *element)
   }
 }
 
+/**
+ * gst_bin_remove_many:
+ * @bin: the bin to remove the elements from
+ * @element_1: the first element to remove from the bin
+ * @...: NULL-terminated list of elements to remove from the bin
+ * 
+ * Remove a list of elements from a bin. Uses #gst_bin_remove.
+ */
+void
+gst_bin_remove_many (GstBin *bin, GstElement *element_1, ...)
+{
+  va_list args;
+
+  g_return_if_fail (GST_IS_BIN (bin));
+  g_return_if_fail (GST_IS_ELEMENT (element_1));
+
+  va_start (args, element_1);
+
+  while (element_1) {
+    gst_bin_remove (bin, element_1);
+    
+    element_1 = va_arg (args, GstElement*);
+  }
+
+  va_end (args);
+}
+
 /**
  * gst_bin_child_state_change:
  * @bin: #GstBin with the child
@@ -572,15 +606,35 @@ gst_bin_remove (GstBin *bin, GstElement *element)
  * of a child.
  */
 void
-gst_bin_child_state_change (GstBin *bin, GstElementState oldstate, GstElementState newstate,
-			    GstElement *child)
+gst_bin_child_state_change (GstBin *bin, GstElementState oldstate, 
+                            GstElementState newstate, GstElement *child)
 {
-  gint old_idx = 0, new_idx = 0, i;
+  GstBinClass *bclass;
+  
+  g_return_if_fail (GST_IS_BIN (bin));
+  g_return_if_fail (GST_IS_ELEMENT (child));
 
-  GST_INFO (GST_CAT_STATES, "child %s changed state in bin %s from %s to %s",
+  GST_CAT_INFO (GST_CAT_STATES, "child %s changed state in bin %s from %s to %s",
 	    GST_ELEMENT_NAME (child), GST_ELEMENT_NAME (bin),
 	    gst_element_state_get_name (oldstate), gst_element_state_get_name (newstate));
 
+  bclass = GST_BIN_GET_CLASS (bin);
+
+  if (bclass->child_state_change) {
+    bclass->child_state_change (bin, oldstate, newstate, child);
+  }
+  else {
+    g_warning ("cannot signal state change of child %s to bin %s\n", 
+               GST_ELEMENT_NAME (child), GST_ELEMENT_NAME (bin));
+  }
+}
+
+static void
+gst_bin_child_state_change_func (GstBin *bin, GstElementState oldstate, 
+                                 GstElementState newstate, GstElement *child)
+{
+  gint old_idx = 0, new_idx = 0, i;
+
   while (oldstate >>= 1) old_idx++;
   while (newstate >>= 1) new_idx++;
 
@@ -592,7 +646,7 @@ gst_bin_child_state_change (GstBin *bin, GstElementState oldstate, GstElementSta
     if (bin->child_states[i] != 0) {
       gint state = (1 << i);
       if (GST_STATE (bin) != state) {
-	GST_INFO (GST_CAT_STATES, "bin %s need state change to %s",
+	GST_CAT_INFO (GST_CAT_STATES, "bin %s need state change to %s",
 		  GST_ELEMENT_NAME (bin), gst_element_state_get_name (state));
 	GST_STATE_PENDING (bin) = state;
         GST_UNLOCK (bin);
@@ -629,19 +683,17 @@ gst_bin_change_state (GstElement * element)
   pending = GST_STATE_PENDING (element);
   transition = GST_STATE_TRANSITION (element);
 
-  GST_INFO_ELEMENT (GST_CAT_STATES, element,
-                    "changing childrens' state from %s to %s",
-		    gst_element_state_get_name (old_state),
-                    gst_element_state_get_name (pending));
+  GST_CAT_INFO (GST_CAT_STATES, "[%s]: changing childrens' state from %s to %s",
+		GST_ELEMENT_NAME (element),
+		gst_element_state_get_name (old_state), gst_element_state_get_name (pending));
 
   if (pending == GST_STATE_VOID_PENDING)
     return GST_STATE_SUCCESS;
 
   if (old_state == pending)
   {
-    GST_INFO_ELEMENT (GST_CAT_STATES, element,
-                      "old and pending state are both %s, returning",
-                      gst_element_state_get_name (pending));
+    GST_CAT_INFO (GST_CAT_STATES, "[%s]: old and pending state are both %s, returning",
+                  GST_ELEMENT_NAME (element), gst_element_state_get_name (pending));
     return GST_STATE_SUCCESS;
   }
 
@@ -653,31 +705,28 @@ gst_bin_change_state (GstElement * element)
     child = GST_ELEMENT (children->data);
     children = g_list_next (children);
 
+    if (GST_FLAG_IS_SET (child, GST_ELEMENT_LOCKED_STATE))
+      continue;
+
     old_child_state = GST_STATE (child);
 
     switch (gst_element_set_state (child, pending)) {
       case GST_STATE_FAILURE:
-	GST_STATE_PENDING (element) = GST_STATE_VOID_PENDING;
-	GST_DEBUG (GST_CAT_STATES, "child '%s' failed to go to state %d(%s)",
+	GST_CAT_DEBUG (GST_CAT_STATES, "child '%s' failed to go to state %d(%s)",
 		   GST_ELEMENT_NAME (child), pending, gst_element_state_get_name (pending));
 
 	gst_element_set_state (child, old_child_state);
+	/* FIXME, this is legacy code, a failed state change of a child should
+	 * return a failure in all cases */
 	if (GST_ELEMENT_SCHED (child) == GST_ELEMENT_SCHED (element)) {
-          /* reset to what is was */
+          /* try to reset it to what is was */
           GST_STATE_PENDING (element) = old_state;
-	  
-          /* FIXME: what about the return value ? why recursive ? */
-          /* gst_bin_change_state (element); */
-	  /* see if the old state was set */
-	  if (GST_STATE (element) == old_state) 
-	    return GST_STATE_SUCCESS;
 
-	  /* something wacky happened, we can't even go back */
 	  return GST_STATE_FAILURE;
 	}
 	break;
       case GST_STATE_ASYNC:
-	GST_DEBUG (GST_CAT_STATES, "child '%s' is changing state asynchronously",
+	GST_CAT_DEBUG (GST_CAT_STATES, "child '%s' is changing state asynchronously",
 		   GST_ELEMENT_NAME (child));
 	have_async = TRUE;
 	break;
@@ -686,7 +735,8 @@ gst_bin_change_state (GstElement * element)
     }
   }
 
-  GST_INFO_ELEMENT (GST_CAT_STATES, element, "done changing bin's state from %s to %s, now in %s",
+  GST_CAT_INFO (GST_CAT_STATES, "[%s]: done changing bin's state from %s to %s, now in %s",
+		GST_ELEMENT_NAME (element),
                 gst_element_state_get_name (old_state),
                 gst_element_state_get_name (pending),
                 gst_element_state_get_name (GST_STATE (element)));
@@ -710,7 +760,7 @@ gst_bin_change_state_norecurse (GstBin * bin)
   GstElementStateReturn ret;
 
   if (parent_class->change_state) {
-    GST_DEBUG_ELEMENT (GST_CAT_STATES, bin, "setting bin's own state");
+    GST_CAT_DEBUG (GST_CAT_STATES, "[%s]: setting bin's own state", GST_ELEMENT_NAME (bin));
     ret = parent_class->change_state (GST_ELEMENT (bin));
 
     return ret;
@@ -726,7 +776,7 @@ gst_bin_dispose (GObject * object)
   GList *children, *orig;
   GstElement *child;
 
-  GST_DEBUG (GST_CAT_REFCOUNTING, "dispose");
+  GST_CAT_DEBUG (GST_CAT_REFCOUNTING, "dispose");
 
   if (gst_element_get_state (GST_ELEMENT (object)) == GST_STATE_PLAYING)
     gst_element_set_state (GST_ELEMENT (object), GST_STATE_PAUSED);
@@ -766,7 +816,8 @@ gst_bin_get_by_name (GstBin * bin, const gchar * name)
   g_return_val_if_fail (GST_IS_BIN (bin), NULL);
   g_return_val_if_fail (name != NULL, NULL);
 
-  GST_INFO_ELEMENT (GST_CAT_PARENTAGE, bin, "looking up child element %s", name);
+  GST_CAT_INFO (GST_CAT_PARENTAGE, "[%s]: looking up child element %s", 
+                GST_ELEMENT_NAME (bin), name);
 
   children = bin->children;
   while (children) {
@@ -829,12 +880,60 @@ gst_bin_get_by_name_recurse_up (GstBin * bin, const gchar * name)
 const GList *
 gst_bin_get_list (GstBin * bin)
 {
-  g_return_val_if_fail (bin != NULL, NULL);
   g_return_val_if_fail (GST_IS_BIN (bin), NULL);
 
   return bin->children;
 }
 
+/**
+ * gst_bin_sync_children_state:
+ * @bin: #Gstbin to sync state
+ *
+ * Tries to set the state of the children of this bin to the same state of the
+ * bin by calling gst_element_set_state for each child not already having a
+ * synchronized state. 
+ *
+ * Returns: The worst return value of any gst_element_set_state. So if one child
+ *          returns #GST_STATE_FAILURE while all others return #GST_STATE_SUCCESS
+ *          this function returns #GST_STATE_FAILURE.
+ */
+GstElementStateReturn
+gst_bin_sync_children_state (GstBin *bin)
+{
+  GList *children;
+  GstElement *element;
+  GstElementState state;
+  GstElementStateReturn ret = GST_STATE_SUCCESS;
+
+  g_return_val_if_fail (GST_IS_BIN (bin), GST_STATE_FAILURE);
+
+  state = GST_STATE (bin);
+  children = bin->children;
+  GST_CAT_INFO (GST_CAT_STATES, "syncing state of children with bin \"%s\"'s state %s",
+            GST_ELEMENT_NAME (bin), gst_element_state_get_name (state));
+
+  while (children) {
+    element = GST_ELEMENT (children->data);
+    children = children->next;
+    if (GST_STATE(element) != state) {
+      switch (gst_element_set_state (element, state)) {
+      case GST_STATE_SUCCESS:
+	break;
+      case GST_STATE_ASYNC:
+	if (ret == GST_STATE_SUCCESS)
+	  ret = GST_STATE_ASYNC;
+	break;
+      case GST_STATE_FAILURE:
+	ret = GST_STATE_FAILURE;
+      default:
+        /* make sure gst_element_set_state never returns this */
+        g_assert_not_reached ();
+      }
+    }
+  }
+
+  return ret;
+}
 #ifndef GST_DISABLE_LOADSAVE
 static xmlNodePtr
 gst_bin_save_thyself (GstObject * object, xmlNodePtr parent)
@@ -849,7 +948,8 @@ gst_bin_save_thyself (GstObject * object, xmlNodePtr parent)
 
   childlist = xmlNewChild (parent, NULL, "children", NULL);
 
-  GST_INFO_ELEMENT (GST_CAT_XML, bin, "saving %d children", bin->numchildren);
+  GST_CAT_INFO (GST_CAT_XML, "[%s]: saving %d children", 
+                GST_ELEMENT_NAME (bin), bin->numchildren);
 
   children = bin->children;
   while (children) {
@@ -870,7 +970,7 @@ gst_bin_restore_thyself (GstObject * object, xmlNodePtr self)
 
   while (field) {
     if (!strcmp (field->name, "children")) {
-      GST_INFO_ELEMENT (GST_CAT_XML, GST_ELEMENT (object), "loading children");
+      GST_CAT_INFO (GST_CAT_XML, "[%s]: loading children", GST_ELEMENT_NAME (object));
       childlist = field->xmlChildrenNode;
       while (childlist) {
 	if (!strcmp (childlist->name, "element")) {
@@ -929,36 +1029,26 @@ gst_bin_iterate_func (GstBin * bin)
 gboolean
 gst_bin_iterate (GstBin *bin)
 {
-  GstBinClass *oclass;
-  gboolean running = TRUE;
-
-  GST_DEBUG_ENTER ("(\"%s\")", GST_ELEMENT_NAME (bin));
+  gboolean running;
 
   g_return_val_if_fail (bin != NULL, FALSE);
   g_return_val_if_fail (GST_IS_BIN (bin), FALSE);
 
-  oclass = GST_BIN_GET_CLASS (bin);
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, bin, "starting iteration");
 
   gst_object_ref (GST_OBJECT (bin));
 
-  if (bin->pre_iterate_func)
-    (bin->pre_iterate_func) (bin, bin->pre_iterate_data);
+  running = FALSE;
+  g_signal_emit (G_OBJECT (bin), gst_bin_signals[ITERATE], 0, &running);
 
-  if (oclass->iterate)
-    running = (oclass->iterate) (bin);
-
-  if (bin->post_iterate_func)
-    (bin->post_iterate_func) (bin, bin->post_iterate_data);
-
-  GST_DEBUG_LEAVE ("(\"%s\") %d", GST_ELEMENT_NAME (bin), running);
+  GST_CAT_LOG_OBJECT (GST_CAT_DATAFLOW, bin, "finished iteration");
 
   if (!running) {
     if (GST_STATE (bin) == GST_STATE_PLAYING && 
 	GST_STATE_PENDING (bin) == GST_STATE_VOID_PENDING) {
-      GST_DEBUG_ELEMENT (GST_CAT_DATAFLOW, bin,
-			 "polling for child shutdown after useless iteration");
-      usleep (1);
-      //gst_element_wait_state_change (GST_ELEMENT (bin));
+      GST_CAT_DEBUG (GST_CAT_DATAFLOW, "[%s]: polling for child shutdown after useless iteration",
+	             GST_ELEMENT_NAME (bin));
+      g_usleep (1);
       running = TRUE;
     }
   }
@@ -967,45 +1057,3 @@ gst_bin_iterate (GstBin *bin)
   return running;
 }
 
-/**
- * gst_bin_set_pre_iterate_function:
- * @bin: #Gstbin to attach to
- * @func: callback function to call
- * @user_data: user data to put in the function call
- *
- * Attaches a callback which will be run before every iteration of the bin
- *
- */
-void
-gst_bin_set_pre_iterate_function (GstBin *bin, GstBinPrePostIterateFunction func, gpointer user_data)
-{
-  g_return_if_fail (GST_IS_BIN (bin));
-
-  if (!GST_FLAG_IS_SET (bin, GST_BIN_FLAG_MANAGER))
-    g_warning ("setting pre_iterate on a non MANAGER bin has no effect");
-  
-  bin->pre_iterate_func = func;
-  bin->pre_iterate_data = user_data;
-}
-
-/**
- * gst_bin_set_post_iterate_function:
- * @bin: #Gstbin to attach to
- * @func: callback function to call
- * @user_data: user data to put in the function call
- *
- * Attaches a callback which will be run after every iteration of the bin
- *
- */
-void
-gst_bin_set_post_iterate_function (GstBin *bin, GstBinPrePostIterateFunction func, gpointer user_data)
-{
-  g_return_if_fail (GST_IS_BIN (bin));
-
-  if (!GST_FLAG_IS_SET (bin, GST_BIN_FLAG_MANAGER))
-    g_warning ("setting post_iterate on a non MANAGER bin has no effect");
-
-  bin->post_iterate_func = func;
-  bin->post_iterate_data = user_data;
-}
-
