@@ -108,6 +108,12 @@ typedef enum {
   GST_ELEMENT_SCHEDULER_PRIVATE1,
   GST_ELEMENT_SCHEDULER_PRIVATE2,
 
+  /* ignore state changes from parent */
+  GST_ELEMENT_LOCKED_STATE,
+
+  /* element is in error */
+  GST_ELEMENT_ERROR,
+
   /* use some padding for future expansion */
   GST_ELEMENT_FLAG_LAST		= GST_OBJECT_FLAG_LAST + 16
 } GstElementFlags;
@@ -160,7 +166,7 @@ struct _GstElement {
   GAsyncQueue		*prop_value_queue;
   GMutex		*property_mutex;
 
-  gpointer		dummy[8];
+  GST_OBJECT_PADDING
 };
 
 struct _GstElementClass {
@@ -212,8 +218,7 @@ struct _GstElementClass {
   GstIndex*		(*get_index)		(GstElement *element);
   void			(*set_index)		(GstElement *element, GstIndex *index);
 
-  /* padding */
-  gpointer		dummy[8];
+  GST_CLASS_PADDING
 };
 
 void			gst_element_class_add_pad_template	(GstElementClass *klass, GstPadTemplate *templ);
@@ -225,7 +230,6 @@ void                    gst_element_class_install_std_props	(GstElementClass *kl
 void 			gst_element_default_error		(GObject *object, GstObject *orig, gchar *error);
 
 GType			gst_element_get_type		(void);
-#define			gst_element_destroy(element)	gst_object_destroy (GST_OBJECT (element))
 
 void			gst_element_set_loop_function	(GstElement *element,
 							 GstElementLoopFunction loop);
@@ -258,6 +262,7 @@ GstClock*		gst_element_get_clock 		(GstElement *element);
 void			gst_element_set_clock 		(GstElement *element, GstClock *clock);
 GstClockReturn		gst_element_clock_wait 		(GstElement *element, 
 							 GstClockID id, GstClockTimeDiff *jitter);
+ 
 /* indexs */
 gboolean		gst_element_is_indexable	(GstElement *element);
 void			gst_element_set_index		(GstElement *element, GstIndex *index);
@@ -291,11 +296,13 @@ GList*			gst_element_get_pad_template_list	(GstElement *element);
 GstPadTemplate*		gst_element_get_compatible_pad_template (GstElement *element, GstPadTemplate *compattempl);
 
 gboolean		gst_element_link		(GstElement *src, GstElement *dest);
-gboolean		gst_element_link_many 		(GstElement *element_1, GstElement *element_2, ...);
+gboolean		gst_element_link_many 		(GstElement *element_1, 
+							 GstElement *element_2, ...);
 gboolean		gst_element_link_filtered 	(GstElement *src, GstElement *dest,
 							 GstCaps *filtercaps);
 void			gst_element_unlink 		(GstElement *src, GstElement *dest);
-void			gst_element_unlink_many 	(GstElement *element_1, GstElement *element_2, ...);
+void			gst_element_unlink_many 	(GstElement *element_1, 
+							 GstElement *element_2, ...);
 
 gboolean		gst_element_link_pads		(GstElement *src, const gchar *srcpadname,
 							 GstElement *dest, const gchar *destpadname);
@@ -307,6 +314,8 @@ void			gst_element_unlink_pads		(GstElement *src, const gchar *srcpadname,
 
 const GstEventMask*	gst_element_get_event_masks	(GstElement *element);
 gboolean		gst_element_send_event		(GstElement *element, GstEvent *event);
+gboolean		gst_element_seek		(GstElement *element, GstSeekType seek_type,
+							 guint64 offset);
 const GstQueryType*	gst_element_get_query_types	(GstElement *element);
 gboolean		gst_element_query		(GstElement *element, GstQueryType type,
 			                                 GstFormat *format, gint64 *value);
@@ -319,6 +328,10 @@ void			gst_element_set_eos		(GstElement *element);
 
 void 			gst_element_error 		(GstElement *element, const gchar *error, ...);
 
+gboolean		gst_element_is_locked_state	(GstElement *element);
+void			gst_element_set_locked_state	(GstElement *element, gboolean locked_state);
+gboolean		gst_element_sync_state_with_parent (GstElement *element);
+
 GstElementState         gst_element_get_state           (GstElement *element);
 GstElementStateReturn	gst_element_set_state		(GstElement *element, GstElementState state);
 
@@ -346,6 +359,8 @@ struct _GstElementDetails {
   gchar *version;               /* version of the element */
   gchar *author;                /* who wrote this thing? */
   gchar *copyright;             /* copyright details (year, etc.) */
+
+  GST_STRUCT_PADDING
 };
 
 #define GST_TYPE_ELEMENT_FACTORY 		(gst_element_factory_get_type())
@@ -373,11 +388,13 @@ struct _GstElementFactory {
   GList *padtemplates;
   guint16 numpadtemplates;
 
-  guint16 rank;			/* used by autoplug to prioritise elements to try */
+  GST_OBJECT_PADDING
 };
 
 struct _GstElementFactoryClass {
   GstPluginFeatureClass parent_class;
+
+  GST_CLASS_PADDING
 };
 
 GType 			gst_element_factory_get_type 		(void);
@@ -399,7 +416,8 @@ GstElement*		gst_element_factory_create		(GstElementFactory *factory,
 GstElement*		gst_element_factory_make		(const gchar *factoryname, const gchar *name);
 GstElement*		gst_element_factory_make_or_warn	(const gchar *factoryname, const gchar *name);
 
-void			gst_element_factory_set_rank		(GstElementFactory *factory, guint16 rank);
+#define			gst_element_factory_set_rank(factory, rank) \
+				gst_plugin_feature_set_rank (GST_PLUGIN_FEATURE (factory), rank)
 
 G_END_DECLS
 
