@@ -20,12 +20,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* #define DEBUG_ENABLED */
 #include "gst_private.h"
 
 #include "gstelement.h"
-#include "gstregistry.h"
-#include "gstlog.h"
+#include "gstregistrypool.h"
+#include "gstinfo.h"
+
+#define GST_CAT_DEFAULT GST_CAT_ELEMENT_FACTORY
 
 static void 		gst_element_factory_class_init 		(GstElementFactoryClass *klass);
 static void 		gst_element_factory_init 		(GstElementFactory *factory);
@@ -103,7 +104,7 @@ gst_element_factory_find (const gchar *name)
     return GST_ELEMENT_FACTORY (feature);
 
   /* this should be an ERROR */
-  GST_DEBUG (GST_CAT_ELEMENT_FACTORY,"no such elementfactory \"%s\"", name);
+  GST_DEBUG ("no such elementfactory \"%s\"", name);
   return NULL;
 }
 
@@ -133,9 +134,9 @@ gst_element_factory_cleanup (GstElementFactory *factory)
   padtemplates = factory->padtemplates;
 
   while (padtemplates) {
-    /* We leak the padtemplates, since they're generally not properly
-     * ref'ed by the factories */
-    /* gst_object_unref (GST_OBJECT (oldtempl)); */
+    GstPadTemplate *oldtempl = GST_PAD_TEMPLATE (padtemplates->data);
+     
+    gst_object_unref (GST_OBJECT (oldtempl));
 
     padtemplates = g_list_next (padtemplates);
   }
@@ -167,6 +168,13 @@ gst_element_factory_new (const gchar *name, GType type,
   g_return_val_if_fail (name != NULL, NULL);
   g_return_val_if_fail (type, NULL);
   g_return_val_if_fail (details, NULL);
+  g_return_val_if_fail (details->longname, NULL);
+  g_return_val_if_fail (details->klass, NULL);
+  g_return_val_if_fail (details->license, NULL);
+  g_return_val_if_fail (details->description, NULL);
+  g_return_val_if_fail (details->version, NULL);
+  g_return_val_if_fail (details->author, NULL);
+  g_return_val_if_fail (details->copyright, NULL);
 
   factory = gst_element_factory_find (name);
 
@@ -209,12 +217,13 @@ gst_element_factory_create (GstElementFactory *factory,
 
   g_return_val_if_fail (factory != NULL, NULL);
 
-  if (!gst_plugin_feature_ensure_loaded (GST_PLUGIN_FEATURE (factory)))
+  if (!gst_plugin_feature_ensure_loaded (GST_PLUGIN_FEATURE (factory))) {
+    GST_INFO ("could not load element factory for element \"%s\"", name);
     return NULL;
+  }
 
-  GST_DEBUG (GST_CAT_ELEMENT_FACTORY,
-             "creating element from factory \"%s\" (name \"%s\", type %d)", 
-             GST_PLUGIN_FEATURE_NAME (factory), name, (gint) factory->type);
+  GST_LOG ("creating element from factory \"%s\" (name \"%s\", type %d)", 
+           GST_PLUGIN_FEATURE_NAME (factory), GST_STR_NULL (name), (gint) factory->type);
 
   if (factory->type == 0) {
       g_critical ("Factory for `%s' has no type",
@@ -222,23 +231,25 @@ gst_element_factory_create (GstElementFactory *factory,
       return NULL;
   }
 
-  /* create an instance of the element */
-  element = GST_ELEMENT (g_object_new (factory->type, NULL));
-  g_assert (element != NULL);
-
-  /* attempt to set the elemenfactory class pointer if necessary */
-  oclass = GST_ELEMENT_CLASS (G_OBJECT_GET_CLASS (element));
+  /* attempt to set the elementfactory class pointer if necessary */
+  oclass = GST_ELEMENT_CLASS (g_type_class_ref (factory->type));
   if (oclass->elementfactory == NULL) {
-    GST_DEBUG (GST_CAT_ELEMENT_FACTORY, "class %s", GST_PLUGIN_FEATURE_NAME (factory));
+    GST_DEBUG ("class %s", GST_PLUGIN_FEATURE_NAME (factory));
     oclass->elementfactory = factory;
 
     /* copy pad template pointers to the element class, 
      * allow for custom padtemplates */
     oclass->padtemplates = g_list_concat (oclass->padtemplates, 
-		    g_list_copy (factory->padtemplates));
+                                          g_list_copy (factory->padtemplates));
     oclass->numpadtemplates += factory->numpadtemplates;
   }
 
+  /* create an instance of the element */
+  element = GST_ELEMENT (g_object_new (factory->type, NULL));
+  g_assert (element != NULL);
+
+  g_type_class_unref (oclass);
+
   gst_object_set_name (GST_OBJECT (element), name);
 
   return element;
@@ -264,20 +275,19 @@ gst_element_factory_make (const gchar *factoryname, const gchar *name)
 
   g_return_val_if_fail (factoryname != NULL, NULL);
 
-  GST_DEBUG (GST_CAT_ELEMENT_FACTORY, "gstelementfactory: make \"%s\" \"%s\"", 
-             factoryname, name);
+  GST_LOG ("gstelementfactory: make \"%s\" \"%s\"", 
+           factoryname, GST_STR_NULL (name));
 
   /* gst_plugin_load_element_factory (factoryname); */
   factory = gst_element_factory_find (factoryname);
   if (factory == NULL) {
-    GST_INFO (GST_CAT_ELEMENT_FACTORY,"no such element factory \"%s\"!",
+    GST_INFO ("no such element factory \"%s\"!",
 	      factoryname);
     return NULL;
   }
   element = gst_element_factory_create (factory, name);
   if (element == NULL) {
-    GST_INFO (GST_CAT_ELEMENT_FACTORY,
-	      "couldn't create instance of element factory \"%s\"!",
+    GST_INFO ("couldn't create instance of element factory \"%s\"!",
 	      factoryname);
     return NULL;
   }
@@ -318,7 +328,7 @@ gst_element_factory_make_or_warn (const gchar *factoryname, const gchar *name)
  */
 void
 gst_element_factory_add_pad_template (GstElementFactory *factory,
-			            GstPadTemplate *templ)
+			              GstPadTemplate *templ)
 {
   g_return_if_fail (factory != NULL);
   g_return_if_fail (templ != NULL);
@@ -396,22 +406,6 @@ gst_element_factory_can_sink_caps (GstElementFactory *factory,
   return FALSE;
 }
 
-/**
- * gst_element_factory_set_rank  :
- * @factory: factory to rank
- * @rank: rank value - higher number means more priority rank
- *
- * Specifies a rank for the element so that 
- * autoplugging uses the most appropriate elements.
- *
- */
-void
-gst_element_factory_set_rank (GstElementFactory *factory, guint16 rank)
-{
-  g_return_if_fail (factory != NULL);
-  factory->rank = rank;
-}
-
 static void
 gst_element_factory_unload_thyself (GstPluginFeature *feature)
 {
