@@ -30,31 +30,30 @@ G_BEGIN_DECLS
 
 extern GstElementDetails gst_pipeline_details;
 
-#define GST_TYPE_PIPELINE \
-  (gst_pipeline_get_type())
-#define GST_PIPELINE(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_PIPELINE,GstPipeline))
-#define GST_PIPELINE_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_PIPELINE,GstPipelineClass))
-#define GST_IS_PIPELINE(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_PIPELINE))
-#define GST_IS_PIPELINE_CLASS(obj) \
-  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_PIPELINE))
+#define GST_TYPE_PIPELINE 		(gst_pipeline_get_type ())
+#define GST_PIPELINE(obj) 		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_PIPELINE, GstPipeline))
+#define GST_IS_PIPELINE(obj) 		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_PIPELINE))
+#define GST_PIPELINE_CLASS(klass) 	(G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_PIPELINE, GstPipelineClass))
+#define GST_IS_PIPELINE_CLASS(klass) 	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_PIPELINE))
+#define GST_PIPELINE_GET_CLASS(obj) 	(G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_PIPELINE, GstPipelineClass))
 
 typedef struct _GstPipeline GstPipeline;
 typedef struct _GstPipelineClass GstPipelineClass;
 
 struct _GstPipeline {
-  GstBin bin;
+  GstBin 	 bin;
+
+  GST_OBJECT_PADDING
 };
 
 struct _GstPipelineClass {
   GstBinClass parent_class;
+
+  GST_CLASS_PADDING
 };
 
 GType		gst_pipeline_get_type		(void);
 GstElement*	gst_pipeline_new		(const gchar *name);
-#define		gst_pipeline_destroy(pipeline)	gst_object_destroy(GST_OBJECT(pipeline))
 
 
 G_END_DECLS
