@@ -24,8 +24,8 @@
 
 #include "gstsystemclock.h"
 #include "gstscheduler.h"
-#include "gstlog.h"
-#include "gstregistry.h"
+#include "gstinfo.h"
+#include "gstregistrypool.h"
 
 static void 	gst_scheduler_class_init 	(GstSchedulerClass *klass);
 static void 	gst_scheduler_init 		(GstScheduler *sched);
@@ -90,14 +90,14 @@ gst_scheduler_dispose (GObject *object)
   GstScheduler *sched = GST_SCHEDULER (object);
 
   /* thse lists should all be NULL */
-  GST_DEBUG (0, "scheduler %p dispose %p %p %p",
+  GST_DEBUG ( "scheduler %p dispose %p %p %p",
 		  object,
 		  sched->clock_providers,
 		  sched->clock_receivers,
 		  sched->schedulers);
 
-  gst_object_swap ((GstObject **)&sched->current_clock, NULL);
-  gst_object_swap ((GstObject **)&sched->clock, NULL);
+  gst_object_replace ((GstObject **)&sched->current_clock, NULL);
+  gst_object_replace ((GstObject **)&sched->clock, NULL);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -228,7 +228,7 @@ gst_scheduler_add_element (GstScheduler *sched, GstElement *element)
 
   /* if it's already in this scheduler, don't bother doing anything */
   if (GST_ELEMENT_SCHED (element) == sched) {
-    GST_DEBUG (GST_CAT_SCHEDULING, "element %s already in scheduler %p", 
+    GST_CAT_DEBUG (GST_CAT_SCHEDULING, "element %s already in scheduler %p", 
 		    GST_ELEMENT_NAME (element), sched);
     return;
   }
@@ -238,11 +238,11 @@ gst_scheduler_add_element (GstScheduler *sched, GstElement *element)
 
   if (gst_element_provides_clock (element)) {
     sched->clock_providers = g_list_prepend (sched->clock_providers, element);
-    GST_DEBUG (GST_CAT_CLOCK, "added clock provider %s", GST_ELEMENT_NAME (element));
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "added clock provider %s", GST_ELEMENT_NAME (element));
   }
   if (gst_element_requires_clock (element)) {
     sched->clock_receivers = g_list_prepend (sched->clock_receivers, element);
-    GST_DEBUG (GST_CAT_CLOCK, "added clock receiver %s", GST_ELEMENT_NAME (element));
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "added clock receiver %s", GST_ELEMENT_NAME (element));
   }
 
   gst_element_set_scheduler (element, sched);
@@ -307,32 +307,30 @@ gst_scheduler_state_transition (GstScheduler *sched, GstElement *element, gint t
         if (clock)
           gst_clock_reset (clock);
 
-        GST_DEBUG (GST_CAT_CLOCK, "scheduler READY to PAUSED clock is %p (%s)", clock, 
+        GST_CAT_DEBUG (GST_CAT_CLOCK, "scheduler READY to PAUSED clock is %p (%s)", clock, 
 			(clock ? GST_OBJECT_NAME (clock) : "nil"));
 
-        gst_object_swap ((GstObject **)&sched->current_clock, (GstObject *)clock);
+        gst_object_replace ((GstObject **)&sched->current_clock, (GstObject *)clock);
         break;
       }
       case GST_STATE_PAUSED_TO_PLAYING:
       {
         GstClock *clock = gst_scheduler_get_clock (sched);
 
-        GST_DEBUG (GST_CAT_CLOCK, "scheduler PAUSED to PLAYING clock is %p (%s)", clock, 
+        GST_CAT_DEBUG (GST_CAT_CLOCK, "scheduler PAUSED to PLAYING clock is %p (%s)", clock, 
 			(clock ? GST_OBJECT_NAME (clock) : "nil"));
 
-        gst_object_swap ((GstObject **)&sched->current_clock, (GstObject *)clock);
-
-	gst_scheduler_set_clock (sched, sched->current_clock);
-        if (sched->current_clock) {
-          GST_DEBUG (GST_CAT_CLOCK, "enabling clock %p (%s)", sched->current_clock, 
-			GST_OBJECT_NAME (sched->current_clock));
-          gst_clock_set_active (sched->current_clock, TRUE);
+	gst_scheduler_set_clock (sched, clock);
+        if (clock) {
+          GST_CAT_DEBUG (GST_CAT_CLOCK, "enabling clock %p (%s)", clock, 
+			GST_OBJECT_NAME (clock));
+          gst_clock_set_active (clock, TRUE);
 	}
         break;
       }
       case GST_STATE_PLAYING_TO_PAUSED:
         if (sched->current_clock) {
-          GST_DEBUG (GST_CAT_CLOCK, "disabling clock %p (%s)", sched->current_clock, 
+          GST_CAT_DEBUG (GST_CAT_CLOCK, "disabling clock %p (%s)", sched->current_clock, 
 			GST_OBJECT_NAME (sched->current_clock));
           gst_clock_set_active (sched->current_clock, FALSE);
 	}
@@ -387,7 +385,7 @@ gst_scheduler_add_scheduler (GstScheduler *sched, GstScheduler *sched2)
   g_return_if_fail (GST_IS_SCHEDULER (sched2));
   g_return_if_fail (sched2->parent_sched == NULL);
 
-  GST_DEBUG (0,"gstscheduler: %p add scheduler %p", sched, sched2);
+  GST_DEBUG ("gstscheduler: %p add scheduler %p", sched, sched2);
 
   gst_object_ref (GST_OBJECT (sched2));
   gst_object_ref (GST_OBJECT (sched));
@@ -417,7 +415,7 @@ gst_scheduler_remove_scheduler (GstScheduler *sched, GstScheduler *sched2)
   g_return_if_fail (GST_IS_SCHEDULER (sched2));
   g_return_if_fail (sched2->parent_sched == sched);
 
-  GST_DEBUG (0,"gstscheduler: %p remove scheduler %p", sched, sched2);
+  GST_DEBUG ("gstscheduler: %p remove scheduler %p", sched, sched2);
 
   sclass = GST_SCHEDULER_GET_CLASS (sched);
 
@@ -563,7 +561,7 @@ gst_scheduler_get_clock (GstScheduler *sched)
   if (GST_FLAG_IS_SET (sched, GST_SCHEDULER_FLAG_FIXED_CLOCK)) {
     clock = sched->clock;  
 
-    GST_DEBUG (GST_CAT_CLOCK, "scheduler using fixed clock %p (%s)", clock, 
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "scheduler using fixed clock %p (%s)", clock, 
 			(clock ? GST_OBJECT_NAME (clock) : "nil"));
   }
   else {
@@ -591,8 +589,8 @@ gst_scheduler_get_clock (GstScheduler *sched)
       clock = gst_system_clock_obtain ();
     }
   }
-  GST_DEBUG (GST_CAT_CLOCK, "scheduler selected clock %p (%s)", clock, 
-		(clock ? GST_OBJECT_NAME (clock) : "nil"));
+  GST_CAT_LOG_OBJECT (GST_CAT_CLOCK, sched, "scheduler selected clock %p (%s)", clock, 
+	              clock ? GST_STR_NULL (GST_OBJECT_NAME (clock)) : "-");
 
   return clock;
 }
@@ -614,9 +612,9 @@ gst_scheduler_use_clock (GstScheduler *sched, GstClock *clock)
 
   GST_FLAG_SET (sched, GST_SCHEDULER_FLAG_FIXED_CLOCK);
 
-  gst_object_swap ((GstObject **)&sched->clock, (GstObject *)clock);
+  gst_object_replace ((GstObject **)&sched->clock, (GstObject *)clock);
 
-  GST_DEBUG (GST_CAT_CLOCK, "scheduler using fixed clock %p (%s)", clock, 
+  GST_CAT_DEBUG (GST_CAT_CLOCK, "scheduler using fixed clock %p (%s)", clock, 
 		(clock ? GST_OBJECT_NAME (clock) : "nil"));
 }
 
@@ -640,12 +638,12 @@ gst_scheduler_set_clock (GstScheduler *sched, GstClock *clock)
   receivers = sched->clock_receivers;
   schedulers = sched->schedulers;
 
-  gst_object_swap ((GstObject **)&sched->current_clock, (GstObject *)clock);
+  gst_object_replace ((GstObject **)&sched->current_clock, (GstObject *)clock);
 
   while (receivers) {
     GstElement *element = GST_ELEMENT (receivers->data);
 
-    GST_DEBUG (GST_CAT_CLOCK, "scheduler setting clock %p (%s) on element %s", clock, 
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "scheduler setting clock %p (%s) on element %s", clock, 
 		(clock ? GST_OBJECT_NAME (clock) : "nil"), GST_ELEMENT_NAME (element));
     
     gst_element_set_clock (element, clock);
@@ -654,7 +652,7 @@ gst_scheduler_set_clock (GstScheduler *sched, GstClock *clock)
   while (schedulers) {
     GstScheduler *scheduler = GST_SCHEDULER (schedulers->data);
 
-    GST_DEBUG (GST_CAT_CLOCK, "scheduler setting clock %p (%s) on scheduler %p", clock, 
+    GST_CAT_DEBUG (GST_CAT_CLOCK, "scheduler setting clock %p (%s) on scheduler %p", clock, 
 		(clock ? GST_OBJECT_NAME (clock) : "nil"), scheduler);
     gst_scheduler_set_clock (scheduler, clock);
     schedulers = g_list_next (schedulers);
@@ -675,9 +673,9 @@ gst_scheduler_auto_clock (GstScheduler *sched)
 
   GST_FLAG_UNSET (sched, GST_SCHEDULER_FLAG_FIXED_CLOCK);
 
-  gst_object_swap ((GstObject **)&sched->clock, NULL);
+  gst_object_replace ((GstObject **)&sched->clock, NULL);
 
-  GST_DEBUG (GST_CAT_CLOCK, "scheduler using automatic clock");
+  GST_CAT_DEBUG (GST_CAT_CLOCK, "scheduler using automatic clock");
 }
 
 /**
@@ -707,8 +705,6 @@ gst_scheduler_clock_wait (GstScheduler *sched, GstElement *element,
     return sclass->clock_wait (sched, element, id, jitter);
   else 
     return gst_clock_id_wait (id, jitter);
-
-  return GST_CLOCK_TIMEOUT;
 }
 
 /**
@@ -723,15 +719,17 @@ gboolean
 gst_scheduler_iterate (GstScheduler *sched)
 {
   GstSchedulerClass *sclass;
+  gboolean res = FALSE;
 
   g_return_val_if_fail (GST_IS_SCHEDULER (sched), FALSE);
 
   sclass = GST_SCHEDULER_GET_CLASS (sched);
 
-  if (sclass->iterate)
-    return sclass->iterate (sched);
+  if (sclass->iterate) {
+    res = sclass->iterate (sched);
+  }
 
-  return FALSE;
+  return res;
 }
 
 
@@ -802,7 +800,7 @@ gst_scheduler_factory_class_init (GstSchedulerFactoryClass *klass)
   factory_parent_class = g_type_class_ref (GST_TYPE_PLUGIN_FEATURE);
 
   if (!_default_name)
-    _default_name = g_strdup ("opt");
+    _default_name = g_strdup (GST_SCHEDULER_DEFAULT_NAME);
 }
 
 static void
@@ -873,7 +871,7 @@ gst_scheduler_factory_find (const gchar *name)
 
   g_return_val_if_fail (name != NULL, NULL);
 
-  GST_DEBUG (0,"gstscheduler: find \"%s\"", name);
+  GST_DEBUG ("gstscheduler: find \"%s\"", name);
 
   feature = gst_registry_pool_find_feature (name, GST_TYPE_SCHEDULER_FACTORY);
 
@@ -897,26 +895,25 @@ gst_scheduler_factory_find (const gchar *name)
 GstScheduler*
 gst_scheduler_factory_create (GstSchedulerFactory *factory, GstElement *parent)
 {
-  GstScheduler *new = NULL;
+  GstScheduler *sched = NULL;
 
   g_return_val_if_fail (factory != NULL, NULL);
   g_return_val_if_fail (GST_IS_ELEMENT (parent), NULL);
-  g_return_val_if_fail (GST_ELEMENT_SCHED (parent) == NULL, NULL);
 
   if (gst_plugin_feature_ensure_loaded (GST_PLUGIN_FEATURE (factory))) {
     g_return_val_if_fail (factory->type != 0, NULL);
 
-    new = GST_SCHEDULER (g_object_new (factory->type, NULL));
-    new->parent = parent;
+    sched = GST_SCHEDULER (g_object_new (factory->type, NULL));
+    sched->parent = parent;
 
-    GST_ELEMENT_SCHED (parent) = new;
+    GST_ELEMENT_SCHED (parent) = sched;
 
     /* let's refcount the scheduler */
-    gst_object_ref (GST_OBJECT (new));
-    gst_object_sink (GST_OBJECT (new));
+    gst_object_ref (GST_OBJECT (sched));
+    gst_object_sink (GST_OBJECT (sched));
   }
 
-  return new;
+  return sched;
 }
 
 /**
