@@ -26,179 +26,8 @@
 
 #include "gst_private.h"
 #include "gstutils.h"
-#include "gstlog.h"
-
-#include "gstextratypes.h"
-
-#define ZERO(mem) memset(&mem, 0, sizeof(mem))
-
-/**
- * gst_util_get_int_arg:
- * @object: the object to query
- * @argname: the name of the argument
- *
- * Retrieves a property of an object as an integer.
- *
- * Returns: the property of the object
- */
-gint
-gst_util_get_int_arg (GObject * object, const gchar * argname)
-{
-  GValue value;
-
-  ZERO (value);
-  g_value_init (&value, G_TYPE_INT);
-  g_object_get_property (G_OBJECT (object), argname, &value);
-
-  return g_value_get_int (&value);
-}
-
-/**
- * gst_util_get_bool_arg:
- * @object: the object to query
- * @argname: the name of the argument
- *
- * Retrieves a property of an object as a boolean.
- *
- * Returns: the property of the object
- */
-gint
-gst_util_get_bool_arg (GObject * object, const gchar * argname)
-{
-  GValue value;
-
-  ZERO (value);
-  g_value_init (&value, G_TYPE_BOOLEAN);
-  g_object_get_property (G_OBJECT (object), argname, &value);
-
-  return g_value_get_boolean (&value);
-}
-
-/**
- * gst_util_get_long_arg:
- * @object: the object to query
- * @argname: the name of the argument
- *
- * Retrieves a property of an object as a long.
- *
- * Returns: the property of the object
- */
-glong
-gst_util_get_long_arg (GObject * object, const gchar * argname)
-{
-  GValue value;
-
-  ZERO (value);
-  g_value_init (&value, G_TYPE_LONG);
-  g_object_get_property (G_OBJECT (object), argname, &value);
-
-  return g_value_get_long (&value);
-}
-
-/**
- * gst_util_get_int64_arg:
- * @object: the object to query
- * @argname: the name of the argument
- *
- * Retrieves a property of an object as an int64.
- *
- * Returns: the property of the object
- */
-gint64
-gst_util_get_int64_arg (GObject *object, const gchar *argname)
-{
-  GValue value;
-
-  ZERO (value);
-  g_value_init (&value, G_TYPE_INT64);
-  g_object_get_property (G_OBJECT (object), argname, &value);
-
-  return g_value_get_int64 (&value);
-}
-
-/**
- * gst_util_get_float_arg:
- * @object: the object to query
- * @argname: the name of the argument
- *
- * Retrieves a property of an object as a float.
- *
- * Returns: the property of the object
- */
-gfloat
-gst_util_get_float_arg (GObject * object, const gchar * argname)
-{
-  GValue value;
-
-  ZERO (value);
-  g_value_init (&value, G_TYPE_FLOAT);
-  g_object_get_property (G_OBJECT (object), argname, &value);
-
-  return g_value_get_float (&value);
-}
-
-/**
- * gst_util_get_double_arg:
- * @object: the object to query
- * @argname: the name of the argument
- *
- * Retrieves a property of an object as a double.
- *
- * Returns: the property of the object
- */
-gdouble
-gst_util_get_double_arg (GObject * object, const gchar * argname)
-{
-  GValue value;
-
-  ZERO (value);
-  g_value_init (&value, G_TYPE_DOUBLE);
-  g_object_get_property (G_OBJECT (object), argname, &value);
-
-  return g_value_get_double (&value);
-}
-
-/**
- * gst_util_get_string_arg:
- * @object: the object to query
- * @argname: the name of the argument
- *
- * Retrieves a property of an object as a string.
- *
- * Returns: the property of the object
- */
-const gchar *
-gst_util_get_string_arg (GObject * object, const gchar * argname)
-{
-  GValue value;
-
-  ZERO (value);
-  g_value_init (&value, G_TYPE_STRING);
-  g_object_get_property (G_OBJECT (object), argname, &value);
-
-  return g_value_get_string (&value);	/* memleak? */
-}
-
-/**
- * gst_util_get_pointer_arg:
- * @object: the object to query
- * @argname: the name of the argument
- *
- * Retrieves a property of an object as a pointer.
- *
- * Returns: the property of the object
- */
-gpointer
-gst_util_get_pointer_arg (GObject * object, const gchar * argname)
-{
-  GValue value;
-
-  ZERO (value);
-  g_value_init (&value, G_TYPE_POINTER);
-  g_object_get_property (G_OBJECT (object), argname, &value);
-
-  return g_value_get_pointer (&value);
-}
+#include "gsturitype.h"
+#include "gstinfo.h"
 
 /**
  * gst_util_dump_mem:
@@ -208,34 +37,33 @@ gst_util_get_pointer_arg (GObject * object, const gchar * argname)
  * Dumps the memory block into a hex representation. Useful for debugging.
  */
 void
-gst_util_dump_mem (guchar * mem, guint size)
+gst_util_dump_mem (guchar *mem, guint size)
 {
   guint i, j;
+  GString *string = g_string_sized_new (50);
+  GString *chars = g_string_sized_new (18);
 
   i = j = 0;
   while (i < size) {
-    if (j == 0) {
-      if (i != 0) {
-	guint k;
+    if (g_ascii_isprint (mem[i]))
+      g_string_append_printf (chars, "%c", mem[i]);
+    else 
+      g_string_append_printf (chars, ".");
 
-	for (k = i - 16; k < i; k++) {
-          if (mem[k]>'a' && mem[k] < 'Z')
-            g_print ("%c", mem[k]);
-	  else 
-            g_print (".");
-	}
-        g_print ("\n");
-      }
-      g_print ("%08x (%p): ", i, mem+i);
-      j = 15;
-    }
-    else {
-      j--;
-    }
-    g_print ("%02x ", mem[i]);
+    g_string_append_printf (string, "%02x ", mem[i]);
+
+    j++;
     i++;
+
+    if (j == 16 || i == size) {
+      g_print ("%08x (%p): %-48.48s %-16.16s\n", i-j, mem+i-j, string->str, chars->str);
+      g_string_set_size (string, 0);
+      g_string_set_size (chars, 0);
+      j = 0;
+    }
   }
-  g_print ("\n");
+  g_string_free (string, TRUE);
+  g_string_free (chars, TRUE);
 }
 
 
@@ -254,7 +82,7 @@ gst_util_set_value_from_string(GValue *value, const gchar *value_str)
 	g_return_if_fail(value != NULL);
 	g_return_if_fail(value_str != NULL);
 	
-	GST_DEBUG(GST_CAT_PARAMS, "parsing '%s' to type %s", value_str, g_type_name(G_VALUE_TYPE(value)));
+	GST_CAT_DEBUG (GST_CAT_PARAMS, "parsing '%s' to type %s", value_str, g_type_name(G_VALUE_TYPE(value)));
 
 	switch (G_VALUE_TYPE(value)) {
 		case G_TYPE_STRING:
@@ -341,7 +169,7 @@ gst_util_set_object_arg (GObject * object, const gchar * name, const gchar * val
       return;
     }
 
-    GST_DEBUG (0, "paramspec->flags is %d, paramspec->value_type is %d",
+    GST_DEBUG ( "paramspec->flags is %d, paramspec->value_type is %d",
 	       paramspec->flags, (gint) paramspec->value_type);
 
     if (paramspec->flags & G_PARAM_WRITABLE) {
@@ -381,7 +209,7 @@ gst_util_set_object_arg (GObject * object, const gchar * name, const gchar * val
 	case G_TYPE_BOOLEAN:{
 	  gboolean i = FALSE;
 
-	  if (!strncmp ("true", value, 4))
+	  if (!g_strncasecmp ("true", value, 4))
 	    i = TRUE;
 	  g_object_set (G_OBJECT (object), name, i, NULL);
 	  break;
@@ -421,7 +249,7 @@ gst_util_set_object_arg (GObject * object, const gchar * name, const gchar * val
 	    sscanf (value, "%d", &i);
 	    g_object_set (G_OBJECT (object), name, i, NULL);
 	  }
-	  else if (paramspec->value_type == GST_TYPE_FILENAME) {
+	  else if (paramspec->value_type == GST_TYPE_URI) {
 	    g_object_set (G_OBJECT (object), name, value, NULL);
 	  }
 	  break;
@@ -476,7 +304,7 @@ gst_print_props (GString *buf, gint indent, GList *props, gboolean showname)
       string_append_indent (buf, 2 + width - strlen (name));
     }
 
-    type = gst_props_entry_get_type (prop);
+    type = gst_props_entry_get_props_type (prop);
     switch (type) {
       case GST_PROPS_INT_TYPE:
       {
@@ -637,31 +465,3 @@ gst_print_element_args (GString * buf, gint indent, GstElement * element)
 
   g_free (specs);
 }
-
-/**
- * gst_util_has_arg:
- * @object: an object
- * @argname: a property it might have
- * @arg_type: the type of the argument it should have
- * 
- * Determines whether this @object has a property of name
- * @argname and of type @arg_type
- * 
- * Return value: TRUE if it has the prop, else FALSE
- **/
-gboolean
-gst_util_has_arg (GObject *object, const gchar *argname, GType arg_type)
-{
-  GParamSpec *pspec;
-
-  pspec = g_object_class_find_property (
-	  G_OBJECT_GET_CLASS (object), argname);
-
-  if (!pspec)
-    return FALSE;
-
-  if (pspec->value_type != arg_type)
-    return FALSE;
-
-  return TRUE;
-}
